package javawrapper;

/**
 * Manages analysis behaviors and data.  Analysis behaviors are those whose
 * only purpose is to calculate something for output; they don't change model
 * state.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>October 28, 2004:  Created (LEM)
 * <br>February 1, 2004:  Added biomass behavior (LEM)
 * <br>March 23, 2005:  Added dimension analysis behavior (LEM)
 * <br>March 28, 2005:  Added bole volume behavior (LEM)
 * <br>January 11, 2006:  Added tree age calculator behavior (LEM)
 * <br>March 10, 2006:  Added merchantable timber value behavior (LEM)
 * <br>March 15, 2006:  Added carbon value behavior (LEM)
 */

public class AnalysisBehaviors
    extends BehaviorTypeBase {

  /**Volume calculator - a in the taper equation (also known as a0)*/
  protected ModelVector mp_fTaperA = new ModelVector(
      "Taper Equation Initial Multiplier (a0)",
      "vo_taperA", "vo_taVal", 0, ModelVector.FLOAT);

  /**Volume calculator - b in the taper equation (also known as a1)*/
  protected ModelVector mp_fTaperB = new ModelVector(
      "Taper Equation DBH Exponent (a1)",
      "vo_taperB", "vo_tbVal", 0, ModelVector.FLOAT);

  /**Volume calculator - c in the taper equation (also known as a2)*/
  protected ModelVector mp_fTaperC = new ModelVector(
      "Taper Equation Height Exponent (a2)",
      "vo_taperC", "vo_tcVal", 0, ModelVector.FLOAT);

  /**Volume calculator - d in the taper equation (also known as b1)*/
  protected ModelVector mp_fTaperD = new ModelVector(
      "Taper Equation X Exponent 1 (b1)",
      "vo_taperD", "vo_tdVal", 0, ModelVector.FLOAT);

  /**Volume calculator - f in the taper equation (also known as b2)*/
  protected ModelVector mp_fTaperF = new ModelVector(
      "Taper Equation X Exponent 2 (b2)",
      "vo_taperF", "vo_tfVal", 0, ModelVector.FLOAT);

  /**Volume calculator - g in the taper equation (also known as b3)*/
  protected ModelVector mp_fTaperG = new ModelVector(
      "Taper Equation X Exponent 3 (b3)",
      "vo_taperG", "vo_tgVal", 0, ModelVector.FLOAT);

  /**Volume calculator - i in the taper equation (also known as b4)*/
  protected ModelVector mp_fTaperI = new ModelVector(
      "Taper Equation X Exponent 4 (b4)",
      "vo_taperI", "vo_tiVal", 0, ModelVector.FLOAT);

  /**Volume calculator - j in the taper equation (also known as b5)*/
  protected ModelVector mp_fTaperJ = new ModelVector(
      "Taper Equation X Exponent 5 (b5)",
      "vo_taperJ", "vo_tjVal", 0, ModelVector.FLOAT);

  /**Volume calculator - k in the taper equation (also known as b6)*/
  protected ModelVector mp_fTaperK = new ModelVector(
      "Taper Equation X Exponent 6 (b6)",
      "vo_taperK", "vo_tkVal", 0, ModelVector.FLOAT);

  /**Volume calculator - a for the diameter-outside-bark (also known as a1)*/
  protected ModelVector mp_fBarkA = new ModelVector(
      "Diameter-Outside-Bark Constant (a1)",
      "vo_barkA", "vo_baVal", 0, ModelVector.FLOAT);

  /**Volume calculator - b for the diameter-outside-bark (also known as a2)*/
  protected ModelVector mp_fBarkB = new ModelVector(
      "Diameter-Outside-Bark First Degree Parameter (a2)",
      "vo_barkB", "vo_bbVal", 0, ModelVector.FLOAT);

  /**Volume calculator - c for the diameter-outside-bark (also known as a3)*/
  protected ModelVector mp_fBarkC = new ModelVector(
      "Diameter-Outside-Bark Second Degree Parameter (a3)",
      "vo_barkC", "vo_bcVal", 0, ModelVector.FLOAT);

  /**Dimension analysis - equation ID*/
  protected ModelVector mp_iEquationID = new ModelVector(
      "Dimension Analysis Equation ID",
      "bi_eqID", "bi_eiVal", 0, ModelVector.MODEL_ENUM);

  /**Dimension analysis - DBH units*/
  protected ModelVector mp_iDbhUnits = new ModelVector(
      "Dimension Analysis DBH Units",
      "bi_dbhUnits", "bi_duVal", 0, ModelVector.MODEL_ENUM);

  /**Dimension analysis - biomass units*/
  protected ModelVector mp_iBiomassUnits = new ModelVector(
      "Dimension Analysis Biomass Units",
      "bi_biomassUnits", "bi_buVal", 0, ModelVector.MODEL_ENUM);

  /**Dimension analysis - whether or not to use a correction factor*/
  protected ModelVector mp_iUseCorrectionFactor = new ModelVector(
      "Dimension Analysis: Use Correction Factor?",
      "bi_useCorrectionFactor", "bi_ucfVal", 0, ModelVector.MODEL_ENUM);

  /**Dimension analysis - correction factor value*/
  protected ModelVector mp_fCorrectionFactor = new ModelVector(
      "Dimension Analysis Correction Factor",
      "bi_correctionFactorValue", "bi_cfvVal ", 0, ModelVector.FLOAT);

  /**Dimension analysis - a in the biomass equation*/
  protected ModelVector mp_fBiomassA = new ModelVector(
      "Dimension Analysis Parameter (a)",
      "bi_a", "bi_aVal", 0, ModelVector.FLOAT);

  /**Dimension analysis - b in the biomass equation*/
  protected ModelVector mp_fBiomassB = new ModelVector(
      "Dimension Analysis Parameter (b)",
      "bi_b", "bi_bVal", 0, ModelVector.FLOAT);

  /**Dimension analysis - c in the biomass equation*/
  protected ModelVector mp_fBiomassC = new ModelVector(
      "Dimension Analysis Parameter (c)",
      "bi_c", "bi_cVal", 0, ModelVector.FLOAT);

  /**Dimension analysis - d in the biomass equation*/
  protected ModelVector mp_fBiomassD = new ModelVector(
      "Dimension Analysis Parameter (d)",
      "bi_d", "bi_dVal", 0, ModelVector.FLOAT);

  /**Dimension analysis - e in the biomass equation*/
  protected ModelVector mp_fBiomassE = new ModelVector(
      "Dimension Analysis Parameter (e)",
      "bi_e", "bi_eVal", 0, ModelVector.FLOAT);

  /**Bole volume - b0 in the volume equation*/
  protected ModelVector mp_fBoleVolumeB0 = new ModelVector(
      "Bole Volume Parameter (b0)",
      "an_boleB0", "an_bb0Val", 0, ModelVector.FLOAT);

  /**Bole volume - b1 in the volume equation*/
  protected ModelVector mp_fBoleVolumeB1 = new ModelVector(
      "Bole Volume Parameter (b1)",
      "an_boleB1", "an_bb1Val", 0, ModelVector.FLOAT);

  /**Bole volume - b2 in the volume equation*/
  protected ModelVector mp_fBoleVolumeB2 = new ModelVector(
      "Bole Volume Parameter (b2)",
      "an_boleB2", "an_bb2Val", 0, ModelVector.FLOAT);

  /**Bole volume - b3 in the volume equation*/
  protected ModelVector mp_fBoleVolumeB3 = new ModelVector(
      "Bole Volume Parameter (b3)",
      "an_boleB3", "an_bb3Val", 0, ModelVector.FLOAT);

  /**Bole volume - b4 in the volume equation*/
  protected ModelVector mp_fBoleVolumeB4 = new ModelVector(
      "Bole Volume Parameter (b4)",
      "an_boleB4", "an_bb4Val", 0, ModelVector.FLOAT);

  /**Bole volume - b5 in the volume equation*/
  protected ModelVector mp_fBoleVolumeB5 = new ModelVector(
      "Bole Volume Parameter (b5)",
      "an_boleB5", "an_bb5Val", 0, ModelVector.FLOAT);

  /**Bole volume - form classes*/
  protected ModelVector mp_fBoleVolumeFormClasses = new ModelVector(
      "Bole Volume Form Class, 60% - 100%",
      "an_boleFormClasses", "an_bfcVal", 0, ModelVector.FLOAT);

  /**Merchantable timber value - form classes*/
  protected ModelVector mp_fMerchValueFormClasses = new ModelVector(
      "Merchantable Timber Value Form Class",
      "an_merchValueFormClasses", "an_mvfcVal", 0, ModelVector.MODEL_ENUM);

  /**Merchantable timber value - price per thousand board feet*/
  protected ModelVector mp_fMerchValuePricePer1K = new ModelVector(
      "Merchantable Timber Value Price / 1000 Board Feet",
      "an_merchValuePricePer1KFeet", "an_mvpp1kfVal", 0, ModelVector.FLOAT);

  /**Carbon value - % of biomass that is carbon*/
  protected ModelVector mp_fCarbonValuePercentBiomassCarbon = new ModelVector(
      "Carbon Value - Carbon Amount of Biomass (0-100%)",
      "an_carbonPercentBiomassCarbon", "an_cpbcVal", 0, ModelVector.FLOAT);

  /**Volume calculator - stump height, in cm.  Point at which to start
   * summing trunk volume*/
  protected ModelFloat m_fStumpHeight = new ModelFloat(0,
      "Height to Begin Calculating Trunk Volume, in cm", "vo_stumpHeight");

  /**Volume calculator - minimum usable diameter, in cm.  Point at which to
   * stop summing trunk volume*/
  protected ModelFloat m_fMinUsableDiam = new ModelFloat(0,
      "Minimum Trunk Diameter for Volume Calculations, in cm",
      "vo_minUsableDiam");

  /**Volume calculator - Length of tree trunk volume segments, in m*/
  protected ModelFloat m_fSegmentLength = new ModelFloat(0,
      "Trunk Segment Length for Volume Calculations, in m", "vo_segmentLength");

  /**Carbon value - value of metric ton of carbon*/
  protected ModelFloat m_fCarbonValueCarbonPrice = new ModelFloat(0,
      "Carbon Value - Price Per Metric Ton Carbon", "an_carbonPricePerMetricTonCarbon");

  /**
   * Constructor.
   * @param oManager GUIManager object.
   * @throws ModelException passed through from called functions.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>October 27, 2004:  Created (LEM)
   * <br>February 1, 2004:  Added biomass behavior (LEM)
   * <br>March 23, 3005:  Replaced biomass behavior with dimension analysis
   * (LEMO
   * <br>March 28, 2005:  Added bole volume behavior (LEM)
   * <br>January 11, 2006:  Added tree age calculator behavior (LEM)
   * <br>March 10, 2006: Added merchantable timber value behavior (LEM)
   */
  public AnalysisBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Analysis", "analysis");

    m_sHelpID = "data.analysis_behaviors";

    //Make our child behaviors
    mp_oChildBehaviors = new Behavior[6];

    //Volume calculator behavior - can be edited automatically, must have trees
    mp_oChildBehaviors[0] = new Behavior("tree volume calculator",
                                         "Tree Volume Calculator",
                                         "tree volume calculator", (float) 1);
    mp_oChildBehaviors[0].m_bMustHaveTrees = true;
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[0].mp_oNewTreeDataMembers.add(new DataMember(
        "Tree Volume", "Volume", DataMember.FLOAT));
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperA);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperB);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperC);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperD);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperF);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperG);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperI);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperJ);
    mp_oChildBehaviors[0].AddRequiredData(mp_fTaperK);
    mp_oChildBehaviors[0].AddRequiredData(mp_fBarkA);
    mp_oChildBehaviors[0].AddRequiredData(mp_fBarkB);
    mp_oChildBehaviors[0].AddRequiredData(mp_fBarkC);
    mp_oChildBehaviors[0].AddRequiredData(m_fMinUsableDiam);
    mp_oChildBehaviors[0].AddRequiredData(m_fSegmentLength);
    mp_oChildBehaviors[0].AddRequiredData(m_fStumpHeight);

    //Dimension analysis behavior - can be edited automatically, must have trees
    mp_oChildBehaviors[1] = new Behavior("tree biomass calculator",
                                         "Dimension Analysis",
                                         "tree biomass calculator", (float) 2);
    mp_oChildBehaviors[1].m_bMustHaveTrees = true;
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[1].mp_oNewTreeDataMembers.add(new DataMember(
        "Tree Biomass", "Biomass", DataMember.FLOAT));
    mp_oChildBehaviors[1].AddRequiredData(mp_iEquationID);
    mp_oChildBehaviors[1].AddRequiredData(mp_iDbhUnits);
    mp_oChildBehaviors[1].AddRequiredData(mp_iBiomassUnits);
    mp_oChildBehaviors[1].AddRequiredData(mp_iUseCorrectionFactor);
    mp_oChildBehaviors[1].AddRequiredData(mp_fCorrectionFactor);
    mp_oChildBehaviors[1].AddRequiredData(mp_fBiomassA);
    mp_oChildBehaviors[1].AddRequiredData(mp_fBiomassB);
    mp_oChildBehaviors[1].AddRequiredData(mp_fBiomassC);
    mp_oChildBehaviors[1].AddRequiredData(mp_fBiomassD);
    mp_oChildBehaviors[1].AddRequiredData(mp_fBiomassE);

    //Dimension analysis behavior - can be edited automatically, must have trees
    mp_oChildBehaviors[2] = new Behavior("bole volume",
                                         "Tree Bole Volume Calculator",
                                         "tree bole volume calculator",
                                         (float) 1);
    mp_oChildBehaviors[2].m_bMustHaveTrees = true;
    mp_oChildBehaviors[2].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[2].mp_oNewTreeDataMembers.add(new DataMember(
        "Tree Bole Volume", "Bole Vol", DataMember.FLOAT));
    mp_oChildBehaviors[2].AddRequiredData(mp_fBoleVolumeFormClasses);
    mp_oChildBehaviors[2].AddRequiredData(mp_fBoleVolumeB0);
    mp_oChildBehaviors[2].AddRequiredData(mp_fBoleVolumeB1);
    mp_oChildBehaviors[2].AddRequiredData(mp_fBoleVolumeB2);
    mp_oChildBehaviors[2].AddRequiredData(mp_fBoleVolumeB3);
    mp_oChildBehaviors[2].AddRequiredData(mp_fBoleVolumeB4);
    mp_oChildBehaviors[2].AddRequiredData(mp_fBoleVolumeB5);

    //Tree age calculator behavior - can be edited automatically, must have
    //trees
    mp_oChildBehaviors[3] = new Behavior("Tree Age",
                                         "Tree Age",
                                         "Tree Age Calculator",
                                         (float) 1);
    mp_oChildBehaviors[3].m_bMustHaveTrees = true;
    mp_oChildBehaviors[3].SetCanApplyTo(TreePopulation.SNAG, false);
    //Data members
    mp_oChildBehaviors[3].mp_oNewTreeDataMembers.add(new DataMember(
        "Tree Age", "Tree Age", DataMember.INTEGER));
    //No parameters

    //Merchantable timber value behavior - can be edited automatically, must
    //have trees
    mp_oChildBehaviors[4] = new Behavior("Merchantable Timber Value",
                                         "Merchantable Timber Value",
                                         "Merch Value Calculator",
                                         (float) 1);
    mp_oChildBehaviors[4].m_bMustHaveTrees = true;
    mp_oChildBehaviors[4].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[4].mp_oNewTreeDataMembers.add(new DataMember(
        "Merchantable Value", "Merch Val", DataMember.FLOAT));
    mp_oChildBehaviors[4].AddRequiredData(mp_fMerchValueFormClasses);
    mp_oChildBehaviors[4].AddRequiredData(mp_fMerchValuePricePer1K);

    //Carbon value behavior - can be edited automatically, must
    //have trees
    mp_oChildBehaviors[5] = new Behavior("Carbon Value",
                                         "Carbon Value Calculator",
                                         "Carbon Value Calculator",
                                         (float) 1);
    mp_oChildBehaviors[5].m_bMustHaveTrees = true;
    mp_oChildBehaviors[5].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[5].AddRequiredData(mp_fCarbonValuePercentBiomassCarbon);
    mp_oChildBehaviors[5].AddRequiredData(m_fCarbonValueCarbonPrice);


    mp_oAllData.add(mp_fTaperA);
    mp_oAllData.add(mp_fTaperB);
    mp_oAllData.add(mp_fTaperC);
    mp_oAllData.add(mp_fTaperD);
    mp_oAllData.add(mp_fTaperF);
    mp_oAllData.add(mp_fTaperG);
    mp_oAllData.add(mp_fTaperI);
    mp_oAllData.add(mp_fTaperJ);
    mp_oAllData.add(mp_fTaperK);
    mp_oAllData.add(mp_fBarkA);
    mp_oAllData.add(mp_fBarkB);
    mp_oAllData.add(mp_fBarkC);
    mp_oAllData.add(m_fMinUsableDiam);
    mp_oAllData.add(m_fSegmentLength);
    mp_oAllData.add(m_fStumpHeight);
    mp_oAllData.add(mp_iEquationID);
    mp_oAllData.add(mp_iDbhUnits);
    mp_oAllData.add(mp_iBiomassUnits);
    mp_oAllData.add(mp_iUseCorrectionFactor);
    mp_oAllData.add(mp_fCorrectionFactor);
    mp_oAllData.add(mp_fBiomassA);
    mp_oAllData.add(mp_fBiomassB);
    mp_oAllData.add(mp_fBiomassC);
    mp_oAllData.add(mp_fBiomassD);
    mp_oAllData.add(mp_fBiomassE);
    mp_oAllData.add(mp_fBoleVolumeFormClasses);
    mp_oAllData.add(mp_fBoleVolumeB0);
    mp_oAllData.add(mp_fBoleVolumeB1);
    mp_oAllData.add(mp_fBoleVolumeB2);
    mp_oAllData.add(mp_fBoleVolumeB3);
    mp_oAllData.add(mp_fBoleVolumeB4);
    mp_oAllData.add(mp_fBoleVolumeB5);
    mp_oAllData.add(mp_fMerchValueFormClasses);
    mp_oAllData.add(mp_fMerchValuePricePer1K);
    mp_oAllData.add(mp_fCarbonValuePercentBiomassCarbon);
    mp_oAllData.add(m_fCarbonValueCarbonPrice);
  }

  /**
   * Sets up the enum parameters and grids.
   * @param oPop TreePopulation object.
   * @throws javawrapper.ModelException Doesn't throw this exception.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>October 27, 2004: Submitted in beta version (LEM)
   * <br>March 23, 2005:  Added enum dimension analysis parameters
   * <br>March 10, 2006:  Added enum and grid for merchantable timber value
   * form classes (LEM)
   * <br>March 15, 2006:  Added grid for carbon value (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws javawrapper.ModelException {
    ModelEnum oVal;
    int iNumSpecies = oPop.GetNumberOfSpecies(), i;

    //Set up the equation ID enums vector - values display in reverse order
    if (mp_iEquationID.GetValue().size() == 0) {
      mp_iEquationID.GetValue().clear();
      for (i = 0; i < iNumSpecies; i++) {
        oVal = new ModelEnum(new int[] {9, 8, 7, 6, 5, 4, 3, 2, 1},
                             new String[] {"9", "8", "7", "6", "5", "4", "3",
                             "2", "1"}, "", "");
        oVal.SetValue(1);
        mp_iEquationID.GetValue().add(oVal);
      }
    } else {
      //We may have added species - add enums if appropriate
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iEquationID.GetValue().elementAt(i)) {
          oVal = new ModelEnum(new int[] {9, 8, 7, 6, 5, 4, 3, 2, 1},
                             new String[] {"9", "8", "7", "6", "5", "4", "3",
                             "2", "1"}, "", "");
          oVal.SetValue(1);
          mp_iEquationID.GetValue().remove(i);
          mp_iEquationID.GetValue().add(i, oVal);
        }
      }
    }

    if (mp_iDbhUnits.GetValue().size() == 0) {
      //Set up the DBH units enums vector - values display in reverse order
      mp_iDbhUnits.GetValue().clear();
      for (i = 0; i < iNumSpecies; i++) {
        oVal = new ModelEnum(new int[] {2, 1, 0},
                             new String[] {"in", "cm", "mm"}, "", "");
        oVal.SetValue(0);
        mp_iDbhUnits.GetValue().add(oVal);
      }
    }
    else {
      //We may have added species - add enums if appropriate
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iDbhUnits.GetValue().elementAt(i)) {
          oVal = new ModelEnum(new int[] {2, 1, 0},
                             new String[] {"in", "cm", "mm"}, "", "");
          oVal.SetValue(0);
          mp_iDbhUnits.GetValue().remove(i);
          mp_iDbhUnits.GetValue().add(i, oVal);
        }
      }
    }

    //Set up the biomass units enums vector - values display in reverse order
    if (mp_iBiomassUnits.GetValue().size() == 0) {
      mp_iBiomassUnits.GetValue().clear();
      for (i = 0; i < iNumSpecies; i++) {
        oVal = new ModelEnum(new int[] {2, 1, 0}
                             , new String[] {"lb", "kg", "g"}
                             , "", "");
        mp_iBiomassUnits.GetValue().add(oVal);
      }
    }
    else {
      //We may have added species - add enums if appropriate
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iBiomassUnits.GetValue().elementAt(i)) {
          oVal = new ModelEnum(new int[] {2, 1, 0}
                               , new String[] {"lb", "kg", "g"}
                               , "", "");
          mp_iBiomassUnits.GetValue().remove(i);
          mp_iBiomassUnits.GetValue().add(i, oVal);
        }
      }
    }

    //Set up the use correction factor enums vector - default all to false
    if (mp_iUseCorrectionFactor.GetValue().size() == 0) {
      mp_iUseCorrectionFactor.GetValue().clear();
      for (i = 0; i < iNumSpecies; i++) {
        oVal = new ModelEnum(new int[] {1, 0}
                             , new String[] {"true", "false"}
                             , "", "");
        oVal.SetValue(0);
        mp_iUseCorrectionFactor.GetValue().add(oVal);
      }
    }
    else {
      //We may have added species - add enums if appropriate
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iUseCorrectionFactor.GetValue().elementAt(i)) {
          oVal = new ModelEnum(new int[] {1, 0}
                               , new String[] {"true", "false"}
                               , "", "");
          oVal.SetValue(0);
          mp_iUseCorrectionFactor.GetValue().remove(i);
          mp_iUseCorrectionFactor.GetValue().add(i, oVal);
        }
      }
    }

    //Set up the merchantable timber value form class enums vector - values
    //display in reverse order
    if (mp_fMerchValueFormClasses.GetValue().size() == 0) {
      mp_fMerchValueFormClasses.GetValue().clear();
      for (i = 0; i < iNumSpecies; i++) {
        oVal = new ModelEnum(new int[] {85, 84, 81, 80, 79, 78}
                             , new String[] {"85", "84", "81", "80", "79", "78"}
                             , "", "");
        mp_fMerchValueFormClasses.GetValue().add(oVal);
      }
    }
    else {
      //We may have added species - add enums if appropriate
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_fMerchValueFormClasses.GetValue().elementAt(i)) {
          oVal = new ModelEnum(new int[] {85, 84, 81, 80, 79, 78}
                               , new String[] {"85", "84", "81", "80", "79",
                               "78"}
                               , "", "");
          mp_fMerchValueFormClasses.GetValue().remove(i);
          mp_fMerchValueFormClasses.GetValue().add(i, oVal);
        }
      }
    }

    //Default all correction factors to 1
    if (mp_fCorrectionFactor.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fCorrectionFactor.GetValue().add(new Float(1.0));
      }
    }

    //Set up the value grid for merchantable timber and carbon value
    DataMember[] p_oDataMembers = new DataMember[iNumSpecies];
    Plot p_oPlot = m_oManager.GetPlot();
    mp_oGrids = new Grid[2];

    String sGridName = "Merchantable Timber Value";

    //The accessible data members are a value for each species
    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i] = new DataMember("Value for " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "value_" + i,
                                         DataMember.FLOAT);
    }

    //Create our grid
    mp_oGrids[0] = new Grid(sGridName, p_oDataMembers, null,
                            p_oPlot.GetPlotXLength(), p_oPlot.GetPlotYLength());

    //Assign the grid to the merchantable value behavior
    GetBehaviorByKey("Merchantable Timber Value").AddGrid(mp_oGrids[0]);

    //Carbon value grid
    sGridName = "Carbon Value";
    p_oDataMembers = new DataMember[2 * iNumSpecies];

    //The accessible data members are a carbon amount and carbon value for
    //each species
    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i] = new DataMember("Mg of Carbon for " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "carbon_" + i,
                                         DataMember.FLOAT);
    }

    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i+iNumSpecies] = new DataMember("Carbon Value for " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "value_" + i,
                                         DataMember.FLOAT);
    }


    //Create our grid
    mp_oGrids[1] = new Grid(sGridName, p_oDataMembers, null,
                            p_oPlot.GetPlotXLength(), p_oPlot.GetPlotYLength());

    //Assign the grid to the carbon value behavior
    GetBehaviorByKey("Carbon Value").AddGrid(mp_oGrids[1]);

  }

  /**
   * Changes the species names in the merchantable timber value grid.
   * @param sOldSpecies String Old name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)
   * @param sNewSpecies String New name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)
   * @throws ModelException if there is a problem.
   */
  public void ChangeOfSpeciesName(String sOldSpecies, String sNewSpecies) throws
      ModelException {
    if (mp_oGrids == null || mp_oGrids.length == 0) {
      return;
    }

    sOldSpecies = sOldSpecies.replace('_', ' ');
    sNewSpecies = sNewSpecies.replace('_', ' ');

    DataMember[] p_oDataMembers;
    String sDisplayName;
    int iPos, i, j;
    boolean bChanged;

    //Go through all the grids and look for display names with the species
    //in them
    for (i = 0; i < mp_oGrids.length; i++) {
      bChanged = false;
      p_oDataMembers = mp_oGrids[i].GetDataMembers();
      for (j = 0; j < p_oDataMembers.length; j++) {
        if (p_oDataMembers[j].GetDisplayName().indexOf(sOldSpecies) > -1) {
          sDisplayName = p_oDataMembers[j].GetDisplayName();
          iPos = sDisplayName.indexOf(sOldSpecies);
          sDisplayName = sDisplayName.substring(0, iPos) + sNewSpecies +
              sDisplayName.substring(iPos + sOldSpecies.length());
          p_oDataMembers[j].SetDisplayName(sDisplayName);
          bChanged = true;
        }
      }
      if (bChanged == true) {
        mp_oGrids[i].SetDataMembers(p_oDataMembers);
      }
    }
  }

  /**
   * Validates the data before writing to a parameter file.
   * @throws ModelException if:
   * <ul>
   * <li>Volume calculator is enabled, and either
   * m_fStumpHeight or m_fMinUsableDiam, is less than 0, or m_fSegmentLength is
   * less than or equal to 0;</li>
   * <li>Bole volume calculator is enabled, and a form class is not between
   * 60 and 100.</li>
   * <li>Carbon value calculator is enabled, and the percent of biomass that is
   * carbon is not between 0 and 100.</li>
   * <li>Carbon value calculator is enabled, and the dimension analysis
   * behavior is not enabled.</li>
   * </ul>
   * @param oPop Not used.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>October 27, 2004:  Created (LEM)
   * <br>November 30, 2004:  Made segment length validation to a positive
   * number (LEM)
   * <br>March 28, 2005:  Added bole volume data validation (LEM)
   * <br>March 15, 2006:  Added carbon value data validation (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {

    if (!AnyBehaviorsEnabled()) {
      return;
    }

    //Validate volume calculator parameters
    if (GetBehaviorByKey("tree volume calculator").m_bIsEnabled) {
      MakeSureNonNegative(m_fMinUsableDiam);
      MakeSurePositive(m_fSegmentLength);
      MakeSureNonNegative(m_fStumpHeight);
      int iNumSpecies = oPop.GetNumberOfSpecies();
      MakeSureRightSize(mp_fTaperA, iNumSpecies);
      MakeSureRightSize(mp_fTaperB, iNumSpecies);
      MakeSureRightSize(mp_fTaperC, iNumSpecies);
      MakeSureRightSize(mp_fTaperD, iNumSpecies);
      MakeSureRightSize(mp_fTaperF, iNumSpecies);
      MakeSureRightSize(mp_fTaperG, iNumSpecies);
      MakeSureRightSize(mp_fTaperI, iNumSpecies);
      MakeSureRightSize(mp_fTaperJ, iNumSpecies);
      MakeSureRightSize(mp_fTaperK, iNumSpecies);
      MakeSureRightSize(mp_fBarkA, iNumSpecies);
      MakeSureRightSize(mp_fBarkB, iNumSpecies);
      MakeSureRightSize(mp_fBarkC, iNumSpecies);
    }

    //Validate bole volume parameters - LEM 03-28-05
    Behavior oBeh = GetBehaviorByKey("bole volume");
    if (oBeh.m_bIsEnabled) {
      int iNumSpecies = oPop.GetNumberOfSpecies();
      MakeSureRightSize(mp_fBoleVolumeB0, iNumSpecies);
      MakeSureRightSize(mp_fBoleVolumeB1, iNumSpecies);
      MakeSureRightSize(mp_fBoleVolumeB2, iNumSpecies);
      MakeSureRightSize(mp_fBoleVolumeB3, iNumSpecies);
      MakeSureRightSize(mp_fBoleVolumeB4, iNumSpecies);
      MakeSureRightSize(mp_fBoleVolumeB5, iNumSpecies);
      MakeSureRightSize(mp_fBoleVolumeFormClasses, iNumSpecies);
      boolean[] p_bApplies = oBeh.GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fBoleVolumeFormClasses, p_bApplies, 60, 100);
    }

    //Validate carbon value parameters - LEM 03-15-06
    oBeh = GetBehaviorByKey("Carbon Value");
    if (oBeh.m_bIsEnabled) {
      boolean[] p_bApplies = oBeh.GetWhichSpeciesUsed(oPop);

      //Make sure all values in percent of biomass that is carbon are
      //between 0 and 100
      MakeSureAllAreBounded(mp_fCarbonValuePercentBiomassCarbon, p_bApplies, 0, 100);

      //Make sure that all trees to which this behavior are applied also have
      //dimension analysis applied
      Behavior oDimAnBeh = GetBehaviorByKey("tree biomass calculator");
      if (false == oDimAnBeh.m_bIsEnabled) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", "If you "+
     "want to use the behavior \"" + oBeh.GetDescriptor() + "\", you must " +
     "also use the behavior \"" + oDimAnBeh.GetDescriptor() +
     "\"."));
      }
      SpeciesTypeCombo oCarbonCombo, oDimAnCombo;
      int i, j, iNumCarbonCombos = oBeh.GetNumberOfCombos(),
          iNumDimAnCombos = oDimAnBeh.GetNumberOfCombos();
      boolean bFound;
      for (i = 0; i < iNumCarbonCombos; i++) {
        bFound = false;
        oCarbonCombo = oBeh.GetSpeciesTypeCombo(i);
        for (j = 0; j < iNumDimAnCombos; j++) {
          oDimAnCombo = oDimAnBeh.GetSpeciesTypeCombo(j);
          if (oCarbonCombo.equals(oDimAnCombo)) {
            bFound = true;
            break;
          }
        }
        if (false == bFound) {
          throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", "All trees "+
     "that use the behavior \"" + oBeh.GetDescriptor() + "\" must also use the " +
     oDimAnBeh.GetDescriptor() + "\" behavior."));
        }
      }
    }
  }
}
