package javawrapper;

import java.lang.Math;
import java.io.BufferedWriter;
import org.xml.sax.Attributes;
import java.util.Vector;

/**
* Manages data for the light behaviors.
* <p>Copyright: Copyright (c) 2003 Charles D. Canham</p>
* <p>Company: Institute of Ecosystem Studies</p>
* @author Lora E. Murphy
* @version 1.0
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)
* <br>June 17, 2004:  Added snag light extinction coefficients (LEM)
* <br>January 10, 2005:  Added the GLI Light Map behavior (LEM)
* <br>February 8, 2005:  Added the Gap Light behavior (LEM)
* <br>May 3, 2005:  Added the Storm Light behavior (LEM)
* <br>December 8, 2005:  Added the Density Light behavior (LEM)
* <br>March 3, 2006:  Changed Density Light to Basal Area Light (LEM)
*/

public class LightBehaviors
    extends BehaviorTypeBase {

  /**Species-specific - light transmission coefficient*/
  protected ModelVector mp_fLightTransmissionCoefficient = new ModelVector(
      "Light Transmission Coefficient (0-1)", "li_lightExtinctionCoefficient",
      "li_lecVal", 0, ModelVector.FLOAT, true);

  /**Species-specific - snag age class 1 light transmission coefficient*/
  protected ModelVector mp_fSnagClass1LightTransmissionCoefficient = new
      ModelVector("Snag Age Class 1 Light Transmission Coefficient (0-1)",
                  "li_snag1LightExtinctionCoefficient", "li_s1lecVal", 0,
                  ModelVector.FLOAT, true);

  /**Species-specific - snag age class 2 light transmission coefficient*/
  protected ModelVector mp_fSnagClass2LightTransmissionCoefficient = new
      ModelVector("Snag Age Class 2 Light Transmission Coefficient (0-1)",
                  "li_snag2LightExtinctionCoefficient", "li_s2lecVal", 0,
                  ModelVector.FLOAT, true);

  /**Species-specific - snag age class 3 light transmission coefficient*/
  protected ModelVector mp_fSnagClass3LightTransmissionCoefficient = new
      ModelVector("Snag Age Class 3 Light Transmission Coefficient (0-1)",
                  "li_snag3LightExtinctionCoefficient", "li_s3lecVal", 0,
                  ModelVector.FLOAT, true);

  /**Basal area light - tree type: angiosperm (0) or conifer (1)*/
  protected ModelVector mp_iBasalAreaLightWhatType = new ModelVector(
      "Basal Area Light - Species Type", "li_baTreeType", "li_bttVal", 0, ModelVector.MODEL_ENUM, true);

  /**Beam fraction of global radiation*/
  protected ModelFloat m_fBeamFractionOfGlobalRadiation = new ModelFloat(0,
      "Beam Fraction of Global Radiation", "li_beamFractGlobalRad");

  /**Clear sky transmission coefficient*/
  protected ModelFloat m_fClearSkyTransmissionCoefficient = new ModelFloat(0,
      "Clear Sky Transmission Coefficient", "li_clearSkyTransCoeff");

  /**Minimum sun angle in radians*/
  protected ModelFloat m_fGLIMinSunAngle = new ModelFloat(0,
      "Minimum Solar Angle for GLI Light, in rad", "li_minSunAngle");

  /**Minimum sun angle in radians for quadrat, if different from GLI*/
  protected ModelFloat m_fQuadratMinSunAngle = new ModelFloat(0,
      "Minimum Solar Angle for Quadrat Light, in rad", "li_minSunAngle");

  /**Minimum sun angle in radians for GLI map, if different from others*/
  protected ModelFloat m_fMapMinSunAngle = new ModelFloat(0,
      "Minimum Solar Angle for GLI Map Creator, in rad", "li_minSunAngle");

  /**Minimum sun angle in radians for GLI points, if different from others*/
  protected ModelFloat m_fPointsMinSunAngle = new ModelFloat(0,
      "Minimum Solar Angle for GLI Points Creator, in rad", "li_minSunAngle");

  /**Height at which GLI is calculated in quadrats*/
  protected ModelFloat m_fQuadratLightHeight = new ModelFloat( (float) 0.675, //default
      "Height at Which GLI is Calculated for Quadrats, in meters",
      "li_quadratLightHeight");

  /**Height at which GLI is calculated for GLI Map Creator*/
  protected ModelFloat m_fMapLightHeight = new ModelFloat( 0,
      "Height at Which GLI is Calculated for GLI Map, in meters",
      "li_mapLightHeight");

  /**Maximum shading radius for sail light*/
  protected ModelFloat m_fSailLightMaxShadingRadius = new ModelFloat(0,
      "Sail Light Maximum Shading Neighbor Distance, in meters",
      "li_maxShadingRadius");

  /**Minimum sun angle in degrees*/
  protected ModelFloat m_fSailLightMaskAngle = new ModelFloat(0,
      "Sail Light Minimum Solar Angle, in degrees", "li_skyMaskAngle");

  /**Light transmission coefficient for the light filter*/
  protected ModelFloat m_fFilterLightTransmissionCoefficient = new ModelFloat(0,
      "Light Filter Light Transmission Coefficient",
      "lf_lightExtinctionCoefficient");

  /**Height of light filter, in meters*/
  protected ModelFloat m_fFilterHeight = new ModelFloat(0,
      "Height of Light Filter, in m", "lf_heightOfFilter");

  /**Storm light - max radius of storm neighbors, in meters*/
  protected ModelFloat m_fStmLightMaxRadius = new ModelFloat(0,
      "Storm Light - Max Radius (m) for Damaged Neighbors",
      "li_stormLightRadius");

  /**Storm light - slope of light function*/
  protected ModelFloat m_fStmLightSlope = new ModelFloat(0,
      "Storm Light - Slope of Light Function", "li_stormLightSlope");

  /**Storm light - intercept of light function*/
  protected ModelFloat m_fStmLightIntercept = new ModelFloat(0,
      "Storm Light - Intercept of Light Function", "li_stormLightIntercept");

  /**Storm light - minimum number of trees for full canopy*/
 protected ModelFloat m_fStmLightMinCanopyTrees = new ModelFloat(0,
     "Storm Light - Minimum Trees For Full Canopy", "li_stormLightMinFullCanopy");

  /**Storm light - standard deviation or clumping parameter*/
  protected ModelFloat m_fStmLightRandPar = new ModelFloat(0,
      "Storm Light - Std. Dev. or Clump Parameter", "li_stormLightRandPar");

  /**Basal area light "a" parameter for calculating mean GLI from basal area*/
  protected ModelFloat m_fBasalAreaLightA = new ModelFloat(0,
      "Basal Area Light - Mean GLI \"a\" Parameter", "li_baLightA");

  /**Basal area light conifer "b" parameter for calculating mean GLI from
   * basal area*/
  protected ModelFloat m_fBasalAreaLightConiferB = new ModelFloat(0,
      "Basal Area Light - Conifer \"b\" Parameter", "li_baConiferLightB");

  /**Basal area light conifer "c" parameter for calculating mean GLI from
   * basal area*/
  protected ModelFloat m_fBasalAreaLightConiferC = new ModelFloat(0,
      "Basal Area Light - Conifer \"c\" Parameter", "li_baConiferLightC");

  /**Basal area light angiosperm "b" parameter for calculating mean GLI from
   * basal area*/
  protected ModelFloat m_fBasalAreaLightAngiospermB = new ModelFloat(0,
      "Basal Area Light - Angiosperm \"b\" Parameter", "li_baAngiospermLightB");

  /**Basal area light angiosperm "c" parameter for calculating mean GLI from
   * basal area*/
  protected ModelFloat m_fBasalAreaLightAngiospermC = new ModelFloat(0,
      "Basal Area Light - Angiosperm \"c\" Parameter", "li_baAngiospermLightC");

  /**Basal area light sigma parameter for lognormal PDF*/
  protected ModelFloat m_fBasalAreaLightSigma = new ModelFloat(0,
      "Basal Area Light - Lognormal PDF Sigma", "li_baLightSigma");

  /**Basal area light minimum DBH (cm) for a tree counting towards the
   * basal area*/
  protected ModelFloat m_fBasalAreaLightMinDBH = new ModelFloat(0,
      "Basal Area Light - Minimum DBH for Trees", "li_baLightMinDBH");

  /**Basal area light basal threshold, in square meters, for new trees in grid
   * cell for recalculating GLI*/
  protected ModelFloat m_fBasalAreaLightChangeThreshold = new ModelFloat(0,
      "Basal Area Light - Minimum BA Change for New GLI (m2)",
      "li_baLightChangeThreshold");

  /** GLI points creator - filename of points input file */
  protected ModelString m_sGLIPointsInFile = new ModelString("", "GLI Points Input File", "");

  /** GLI points creator - filename of points output file */
  protected ModelString m_sGLIPointsOutFile = new ModelString("", "GLI Points Output File", "li_GLIPointsFilename");

  /**Storm light - stochasticity*/
  private ModelEnum m_iStmLightStochasticity =
      new ModelEnum(new int[] {0, 1, 2, 3},
                    new String[] {"Deterministic", "Lognormal", "Normal",
                   "Negative binomial"}, "Storm Light - Stochasticity",
                   "li_stormLightStoch");

  /**Storm light - max time (years) for damaged trees to influence*/
  protected ModelInt m_iStmLightMaxDmgTime = new ModelInt(0,
     "Storm Light - Max Years Damaged Trees Affect Light",
     "li_stormLightMaxDmgTime");

  /**Storm light - max time (years) for snags to influence*/
  protected ModelInt m_iStmLightMaxSnagDmgTime = new ModelInt(0,
     "Storm Light - Max Years Snags Affect Light",
     "li_stormLightSnagMaxDmgTime");

  /**Number of azimuth divisions in sky hemisphere for GLI light calculations*/
  protected ModelInt m_iNumGLIAziDiv = new ModelInt(0,
      "Number of Azimuth Sky Divisions for GLI Light Calculations",
      "li_numAziGrids");

  /**Number of altitude divisions in sky hemisphere for GLI light calculations*/
  protected ModelInt m_iNumGLIAltDiv = new ModelInt(0,
      "Number of Altitude Sky Divisions for GLI Light Calculations",
      "li_numAltGrids");

  /**Number of azimuth divisions for quadrat, if different from GLI*/
  protected ModelInt m_iNumQuadratAziDiv = new ModelInt(0,
      "Number of Azimuth Sky Divisions for Quadrat Light Calculations",
      "li_numAziGrids");

  /**Number of azimuth divisions for GLI Map Creator, if different from GLI*/
  protected ModelInt m_iNumMapAziDiv = new ModelInt(0,
      "Number of Azimuth Sky Divisions for GLI Map Creator Calculations",
      "li_numAziGrids");

  /**Number of azimuth divisions for GLI Points Creator, if different from GLI*/
  protected ModelInt m_iNumPointsAziDiv = new ModelInt(0,
      "Number of Azimuth Sky Divisions for GLI Points Creator",
      "li_numAziGrids");

  /**Number of altitude divisions for quadrat, if different from GLI*/
  protected ModelInt m_iNumQuadratAltDiv = new ModelInt(0,
      "Number of Altitude Sky Divisions for Quadrat Light Calculations",
      "li_numAltGrids");

  /**Number of altitude divisions for GLI Map Creator, if different from GLI*/
  protected ModelInt m_iNumMapAltDiv = new ModelInt(0,
      "Number of Altitude Sky Divisions for GLI Map Creator Calculations",
      "li_numAltGrids");

  /**Number of altitude divisions for GLI Points Creator, if different from GLI*/
  protected ModelInt m_iNumPointsAltDiv = new ModelInt(0,
      "Number of Altitude Sky Divisions for GLI Points Creator",
      "li_numAltGrids");

  /**Start of growing season as Julian day*/
  protected ModelInt m_iJulianDayGrowthStarts = new ModelInt(0,
      "First Day of Growing Season",
      "li_julianDayGrowthStarts");

  /**End of growing season as Julian day*/
  protected ModelInt m_iJulianDayGrowthEnds = new ModelInt(0,
      "Last Day of Growing Season",
      "li_julianDayGrowthEnds");

  /**Upper age limit of snag size class 1*/
  protected ModelInt m_iSnagAgeClass1 = new ModelInt(0,
      "Upper Age (Yrs) of Snag Light Transmission Class 1", "li_snagAgeClass1");

  /**Upper age limit of snag size class 2*/
  protected ModelInt m_iSnagAgeClass2 = new ModelInt(0,
      "Upper Age (Yrs) of Snag Light Transmission Class 2", "li_snagAgeClass2");

  protected ModelEnum
      /**Height of fisheye photo - valid values are MID_CROWN and CROWN_TOP*/
      m_iHeightOfFishEyePhoto = new ModelEnum(new int[] {MID_CROWN, CROWN_TOP}
                                              ,
                                              new String[] {"Mid-crown",
                                              "Crown top"}
                                              , "Height of Fisheye Photo",
                                              "li_heightOfFishEyePhoto");

  /**What fraction of the height of a shading neighbor its crown covers -
   * valid values are ALL_HEIGHT or FRAC_HEIGHT*/
  protected ModelEnum m_iCrownFractionOfHeight = new ModelEnum(
      new int[] {ALL_HEIGHT, FRAC_HEIGHT},
      new String[] {"All height", "Part of height"},
      "Calculated Crown Depth", "li_crownFracOfHeight");

  /**Points objects, for GLI points*/
  protected Vector mp_oPoints = new Vector(0);

  /**Value for fisheye photo taken at mid-crown - MUST match the enum
   * value from the C++ code
   */
  public final static int MID_CROWN = 0;

  /**Value for fisheye photo taken at crown top - MUST match the enum
   * value from the C++ code
   */
  public final static int CROWN_TOP = 1;

  /**Value for sail light shading neighbor crown depths being approximated
   * as extending the full height of the tree - MUST match the enum
   * value from the C++ code
   */
  public final static int ALL_HEIGHT = 0;

  /**Value for sail light shading neighbor crown depths having their normal
   * depth - MUST match the enum value from the C++ code
   */
  public final static int FRAC_HEIGHT = 1;

  /**
   * Constructor.
   * @param oManager GUIManager object.
   * @throws ModelException If something goes wrong in setup.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public LightBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Light", null);

    m_sHelpID = "data.light_behaviors";

    m_iHeightOfFishEyePhoto.SetValue(MID_CROWN); //default
    m_iCrownFractionOfHeight.SetValue(ALL_HEIGHT); //default

    //Initialize list of light behaviors
    mp_oChildBehaviors = new Behavior[10];
    //Quadrat light
    mp_oChildBehaviors[0] = new Behavior("quadrat", "Quadrat-based GLI light",
                                         "quadratlight", (float) 1);
    //Data
    mp_oChildBehaviors[0].AddRequiredData(m_fBeamFractionOfGlobalRadiation);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearSkyTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(m_iJulianDayGrowthStarts);
    mp_oChildBehaviors[0].AddRequiredData(m_iJulianDayGrowthEnds);
    mp_oChildBehaviors[0].AddRequiredData(m_iNumQuadratAziDiv);
    mp_oChildBehaviors[0].AddRequiredData(m_iNumQuadratAltDiv);
    mp_oChildBehaviors[0].AddRequiredData(m_fQuadratMinSunAngle);
    mp_oChildBehaviors[0].AddRequiredData(m_fQuadratLightHeight);
    mp_oChildBehaviors[0].AddRequiredData(m_iHeightOfFishEyePhoto);
    mp_oChildBehaviors[0].AddRequiredData(mp_fLightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(m_iSnagAgeClass1);
    mp_oChildBehaviors[0].AddRequiredData(m_iSnagAgeClass2);
    //Data members
    mp_oChildBehaviors[0].mp_oNewTreeDataMembers.add(new DataMember("Light level", "Light", DataMember.FLOAT));

    //GLI light
    mp_oChildBehaviors[1] = new Behavior("gli", "GLI light", "glilight",
                                         (float) 1);
    //Data
    mp_oChildBehaviors[1].AddRequiredData(m_fBeamFractionOfGlobalRadiation);
    mp_oChildBehaviors[1].AddRequiredData(m_fClearSkyTransmissionCoefficient);
    mp_oChildBehaviors[1].AddRequiredData(m_iJulianDayGrowthStarts);
    mp_oChildBehaviors[1].AddRequiredData(m_iJulianDayGrowthEnds);
    mp_oChildBehaviors[1].AddRequiredData(m_iNumGLIAziDiv);
    mp_oChildBehaviors[1].AddRequiredData(m_iNumGLIAltDiv);
    mp_oChildBehaviors[1].AddRequiredData(m_fGLIMinSunAngle);
    mp_oChildBehaviors[1].AddRequiredData(m_iHeightOfFishEyePhoto);
    mp_oChildBehaviors[1].AddRequiredData(mp_fLightTransmissionCoefficient);
    mp_oChildBehaviors[1].AddRequiredData(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oChildBehaviors[1].AddRequiredData(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oChildBehaviors[1].AddRequiredData(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oChildBehaviors[1].AddRequiredData(m_iSnagAgeClass1);
    mp_oChildBehaviors[1].AddRequiredData(m_iSnagAgeClass2);
    //Data members
    mp_oChildBehaviors[1].mp_oNewTreeDataMembers.add(new DataMember("Light level", "Light", DataMember.FLOAT));

    //Sail light
    mp_oChildBehaviors[2] = new Behavior("saillight", "Sail light", "saillight",
                                         (float) 1);
    //Data
    mp_oChildBehaviors[2].AddRequiredData(m_fBeamFractionOfGlobalRadiation);
    mp_oChildBehaviors[2].AddRequiredData(m_fClearSkyTransmissionCoefficient);
    mp_oChildBehaviors[2].AddRequiredData(m_iJulianDayGrowthStarts);
    mp_oChildBehaviors[2].AddRequiredData(m_iJulianDayGrowthEnds);
    mp_oChildBehaviors[2].AddRequiredData(m_fSailLightMaskAngle);
    mp_oChildBehaviors[2].AddRequiredData(m_fSailLightMaxShadingRadius);
    mp_oChildBehaviors[2].AddRequiredData(m_iCrownFractionOfHeight);
    mp_oChildBehaviors[2].AddRequiredData(m_iHeightOfFishEyePhoto);
    mp_oChildBehaviors[2].AddRequiredData(mp_fLightTransmissionCoefficient);
    mp_oChildBehaviors[2].AddRequiredData(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oChildBehaviors[2].AddRequiredData(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oChildBehaviors[2].AddRequiredData(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oChildBehaviors[2].AddRequiredData(m_iSnagAgeClass1);
    mp_oChildBehaviors[2].AddRequiredData(m_iSnagAgeClass2);
    //Data members
    mp_oChildBehaviors[2].mp_oNewTreeDataMembers.add(new DataMember("Light level", "Light", DataMember.FLOAT));


    //Radial light
    mp_oChildBehaviors[3] = new Behavior("radiallight", "Constant full GLI",
                                         "radiallight", (float) 1);
    //Data members
    mp_oChildBehaviors[3].mp_oNewTreeDataMembers.add(new DataMember("Light level", "Light", DataMember.FLOAT));

    mp_oChildBehaviors[4] = new Behavior("light filter",
                                         "Beer's Law light filter",
                                         "light filter", (float) 1);
    //Data
    mp_oChildBehaviors[4].AddRequiredData(m_fFilterHeight);
    mp_oChildBehaviors[4].AddRequiredData(m_fFilterLightTransmissionCoefficient);
    //Data members
    mp_oChildBehaviors[4].mp_oNewTreeDataMembers.add(new DataMember("Light filter respite counter", "lf_count", DataMember.INTEGER));
    mp_oChildBehaviors[4].mp_oNewTreeDataMembers.add(new DataMember("Rooting height", "z", DataMember.INTEGER));

    //GLI map creator
    mp_oChildBehaviors[5] = new Behavior("map", "GLI Map Creator",
                                         "GLI Map Creator", (float) 1);
    //Doesn't need to apply to trees
    mp_oChildBehaviors[5].m_bMustHaveTrees = false;
    //Data
    mp_oChildBehaviors[5].AddRequiredData(m_fBeamFractionOfGlobalRadiation);
    mp_oChildBehaviors[5].AddRequiredData(m_fClearSkyTransmissionCoefficient);
    mp_oChildBehaviors[5].AddRequiredData(m_iJulianDayGrowthStarts);
    mp_oChildBehaviors[5].AddRequiredData(m_iJulianDayGrowthEnds);
    mp_oChildBehaviors[5].AddRequiredData(m_iNumMapAziDiv);
    mp_oChildBehaviors[5].AddRequiredData(m_iNumMapAltDiv);
    mp_oChildBehaviors[5].AddRequiredData(m_fMapMinSunAngle);
    mp_oChildBehaviors[5].AddRequiredData(m_fMapLightHeight);
    mp_oChildBehaviors[5].AddRequiredData(mp_fLightTransmissionCoefficient);
    mp_oChildBehaviors[5].AddRequiredData(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oChildBehaviors[5].AddRequiredData(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oChildBehaviors[5].AddRequiredData(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oChildBehaviors[5].AddRequiredData(m_iSnagAgeClass1);
    mp_oChildBehaviors[5].AddRequiredData(m_iSnagAgeClass2);

    //GLI points file creator
    mp_oChildBehaviors[6] = new Behavior("points", "GLI Points File Creator",
                                         "GLI Point Creator", (float) 1);
    //Doesn't need to apply to trees
    mp_oChildBehaviors[6].m_bMustHaveTrees = false;
    //Data
    mp_oChildBehaviors[6].AddRequiredData(m_fBeamFractionOfGlobalRadiation);
    mp_oChildBehaviors[6].AddRequiredData(m_fClearSkyTransmissionCoefficient);
    mp_oChildBehaviors[6].AddRequiredData(m_iJulianDayGrowthStarts);
    mp_oChildBehaviors[6].AddRequiredData(m_iJulianDayGrowthEnds);
    mp_oChildBehaviors[6].AddRequiredData(m_iNumPointsAziDiv);
    mp_oChildBehaviors[6].AddRequiredData(m_iNumPointsAltDiv);
    mp_oChildBehaviors[6].AddRequiredData(m_fPointsMinSunAngle);
    mp_oChildBehaviors[6].AddRequiredData(m_sGLIPointsInFile);
    mp_oChildBehaviors[6].AddRequiredData(m_sGLIPointsOutFile);
    mp_oChildBehaviors[6].AddRequiredData(mp_fLightTransmissionCoefficient);
    mp_oChildBehaviors[6].AddRequiredData(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oChildBehaviors[6].AddRequiredData(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oChildBehaviors[6].AddRequiredData(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oChildBehaviors[6].AddRequiredData(m_iSnagAgeClass1);
    mp_oChildBehaviors[6].AddRequiredData(m_iSnagAgeClass2);

    //Gap light
    mp_oChildBehaviors[7] = new Behavior("gap light", "Gap Light",
                                         "Gap Light", (float) 1);

    //Storm light
    mp_oChildBehaviors[8] = new Behavior("storm light", "Storm Light",
                                         "Storm Light", (float) 1);
    //Doesn't need to apply to trees
    mp_oChildBehaviors[8].m_bMustHaveTrees = false;
    //Data
    mp_oChildBehaviors[8].AddRequiredData(m_fStmLightMaxRadius);
    mp_oChildBehaviors[8].AddRequiredData(m_fStmLightSlope);
    mp_oChildBehaviors[8].AddRequiredData(m_fStmLightIntercept);
    mp_oChildBehaviors[8].AddRequiredData(m_fStmLightMinCanopyTrees);
    mp_oChildBehaviors[8].AddRequiredData(m_iStmLightMaxDmgTime);
    mp_oChildBehaviors[8].AddRequiredData(m_iStmLightMaxSnagDmgTime);
    mp_oChildBehaviors[8].AddRequiredData(m_iStmLightStochasticity);
    mp_oChildBehaviors[8].AddRequiredData(m_fStmLightRandPar);


    //Basal area light
    mp_oChildBehaviors[9] = new Behavior("basal area light", "Basal Area Light",
                                         "Basal Area Light", (float) 1);
    //Data
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightA);
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightAngiospermB);
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightAngiospermC);
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightConiferB);
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightConiferC);
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightChangeThreshold);
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightMinDBH);
    mp_oChildBehaviors[9].AddRequiredData(m_fBasalAreaLightSigma);
    mp_oChildBehaviors[9].AddRequiredData(mp_iBasalAreaLightWhatType);
    //Data members
    mp_oChildBehaviors[9].mp_oNewTreeDataMembers.add(new DataMember("Light level", "Light", DataMember.FLOAT));


    //All data objects
    mp_oAllData.add(m_fBeamFractionOfGlobalRadiation);
    mp_oAllData.add(m_fClearSkyTransmissionCoefficient);
    mp_oAllData.add(m_iJulianDayGrowthStarts);
    mp_oAllData.add(m_iJulianDayGrowthEnds);
    mp_oAllData.add(m_fSailLightMaskAngle);
    mp_oAllData.add(m_fSailLightMaxShadingRadius);
    mp_oAllData.add(m_iCrownFractionOfHeight);
    mp_oAllData.add(m_iHeightOfFishEyePhoto);
    mp_oAllData.add(mp_fLightTransmissionCoefficient);
    mp_oAllData.add(m_iNumGLIAziDiv);
    mp_oAllData.add(m_iNumGLIAltDiv);
    mp_oAllData.add(m_fGLIMinSunAngle);
    mp_oAllData.add(m_iNumQuadratAziDiv);
    mp_oAllData.add(m_iNumQuadratAltDiv);
    mp_oAllData.add(m_fQuadratMinSunAngle);
    mp_oAllData.add(m_fQuadratLightHeight);
    mp_oAllData.add(m_fFilterHeight);
    mp_oAllData.add(m_fFilterLightTransmissionCoefficient);
    mp_oAllData.add(mp_fSnagClass1LightTransmissionCoefficient);
    mp_oAllData.add(mp_fSnagClass2LightTransmissionCoefficient);
    mp_oAllData.add(mp_fSnagClass3LightTransmissionCoefficient);
    mp_oAllData.add(m_iSnagAgeClass1);
    mp_oAllData.add(m_iSnagAgeClass2);
    mp_oAllData.add(m_iNumMapAziDiv);
    mp_oAllData.add(m_iNumMapAltDiv);
    mp_oAllData.add(m_fMapMinSunAngle);
    mp_oAllData.add(m_fMapLightHeight);
    mp_oAllData.add(m_fStmLightMaxRadius);
    mp_oAllData.add(m_fStmLightSlope);
    mp_oAllData.add(m_fStmLightIntercept);
    mp_oAllData.add(m_fStmLightMinCanopyTrees);
    mp_oAllData.add(m_iStmLightMaxDmgTime);
    mp_oAllData.add(m_iStmLightMaxSnagDmgTime);
    mp_oAllData.add(m_iStmLightStochasticity);
    mp_oAllData.add(m_fStmLightRandPar);
    mp_oAllData.add(m_iNumPointsAziDiv);
    mp_oAllData.add(m_iNumPointsAltDiv);
    mp_oAllData.add(m_fPointsMinSunAngle);
    mp_oAllData.add(m_sGLIPointsOutFile);
    mp_oAllData.add(m_sGLIPointsInFile);
    mp_oAllData.add(m_fBasalAreaLightA);
    mp_oAllData.add(m_fBasalAreaLightAngiospermB);
    mp_oAllData.add(m_fBasalAreaLightAngiospermC);
    mp_oAllData.add(m_fBasalAreaLightConiferB);
    mp_oAllData.add(m_fBasalAreaLightConiferC);
    mp_oAllData.add(m_fBasalAreaLightChangeThreshold);
    mp_oAllData.add(m_fBasalAreaLightMinDBH);
    mp_oAllData.add(m_fBasalAreaLightSigma);
    mp_oAllData.add(mp_iBasalAreaLightWhatType);

    //Grids
    mp_oGrids = new Grid[5];
    //Quadrat grid
    String sGridName = "Quadrat GLI";

    //Create the data member
    DataMember[] p_oDataMembers = new DataMember[1];

    p_oDataMembers[0] = new DataMember("GLI", "GLI", DataMember.FLOAT);
    mp_oGrids[0] = new Grid(sGridName, p_oDataMembers, null, 2, 2);

    //Assign to quadrat light
    GetBehaviorByKey("quadrat").AddGrid(mp_oGrids[0]);

    //GLI map
    sGridName = "GLI Map";

    //Data member's the same
    mp_oGrids[1] = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Assign to GLI Map Creator
    GetBehaviorByKey("map").AddGrid(mp_oGrids[1]);

    //Gap light
    sGridName = "Gap Light";

    //Data member
    p_oDataMembers = null;
    p_oDataMembers = new DataMember[1];
    p_oDataMembers[0] = new DataMember("Is Gap", "Is Gap", DataMember.BOOLEAN);
    mp_oGrids[2] = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Assign to Gap Light
    GetBehaviorByKey("gap light").AddGrid(mp_oGrids[2]);

    //Storm light
    sGridName = "Storm Light";

    //Data member
    p_oDataMembers = null;
    p_oDataMembers = new DataMember[1];
    p_oDataMembers[0] = new DataMember("Light", "Light", DataMember.FLOAT);
    mp_oGrids[3] = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Assign to Storm Light
    GetBehaviorByKey("storm light").AddGrid(mp_oGrids[3]);


    //Basal area light
    sGridName = "Basal Area Light";

    //Data member
    p_oDataMembers = null;
    p_oDataMembers = new DataMember[3];
    p_oDataMembers[0] = new DataMember("Light", "Light", DataMember.FLOAT);
    p_oDataMembers[1] = new DataMember("Conifer Basal Area", "Con BA", DataMember.FLOAT);
    p_oDataMembers[2] = new DataMember("Angiosperm Basal Area", "Ang BA", DataMember.FLOAT);
    mp_oGrids[4] = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Assign to Basal Area Light
    GetBehaviorByKey("basal area light").AddGrid(mp_oGrids[4]);

  }

  /**
   * Adds the points from a tab-delimited text GLI points file.  Any existing
   * points are overwritten.
   * @param sFileName File name of points file
   * @throws ModelException if:
   * <ul>
   * <li>A GLI coordinate is outside the plot</li>
   * <li>There's a negative GLI height</li>
   * </ul>
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 11, 2005: Created (LEM)
   */
  public void AddGLIPointsFile(String sFileName) throws
      ModelException {
    try {
      Plot oPlot = m_oManager.GetPlot();
      java.io.FileReader oIn = new java.io.FileReader(sFileName);
      Vector oLine = new Vector(0), //one line of data
          oNewPoints = new Vector(0); //the trees contained in this file
      float fX, fY, fHeight;
      int i;

      //Skip the first line - just column headers
      ModelFileFunctions.SkipLine(oIn);

      oLine = ModelFileFunctions.ReadLine(oIn);
      while (oLine.size() > 0) {

        //Verify that all data is present
        if (oLine.size() != 3) {
          throw (new ModelException(ErrorGUI.BAD_FILE, "JAVA",
                                    "The file \"" + sFileName +
                                    "\" has an invalid format."));
        }

        //Get X, Y, and height
        fX = new Float( (String) oLine.elementAt(0)).floatValue();
        fY = new Float( (String) oLine.elementAt(1)).floatValue();
        fHeight = new Float( (String) oLine.elementAt(2)).floatValue();

        Points oPoint = new Points(fX, fY, fHeight, oPlot);
        oNewPoints.add(oPoint);
        oLine = ModelFileFunctions.ReadLine(oIn);
      }
      oIn.close();

      //Add the new points
      mp_oPoints = null;
      mp_oPoints = new Vector(0);

      for (i = 0; i < oNewPoints.size(); i++) {
        mp_oPoints.add(oNewPoints.elementAt(i));
      }
      oNewPoints = null;
    }
    catch (java.io.IOException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_FILE;
      oErr.sCallingFunction = "AddGLIPoints";
      oErr.sMessage = sFileName;
      throw oErr;
    }
  }

  /**
   * Resets the points list.
   * @param oPop Tree population
   * @throws ModelException Not thrown.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    mp_oPoints.clear();

    ModelEnum oEnum;
    int iNumSpecies = oPop.GetNumberOfSpecies(), i;
    for (i = 0; i < iNumSpecies; i++) {
      oEnum = new ModelEnum(new int[] {1, 0},
                            new String[] {"Conifer", "Angiosperm"},
                            "Species type", "");
      oEnum.SetValue("Conifer");
      mp_iBasalAreaLightWhatType.GetValue().add(oEnum);
    }

  }

  /**
   * Overridden to make sure that only light values that apply to light
   * behaviors are set.  There are other behaviors with light values (like
   * Puerto Rico establishment), and this will make sure that those parameters
   * are not "stolen" by light.
   * @param sXMLTag Parent XML tag of data vector whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param p_oData Vector of data values appropriate to the data type
   * @param p_sChildXMLTags The XML tags of the child elements
   * @param p_bAppliesTo Array of booleans saying which of the vector values
   * should be set.  This is important in the case of species-specifics - the
   * vector index is the species number but not all species are set.
   * @param oParentAttributes Attributes of parent tag.  May be useful when
   * overridding this for unusual tags.
   * @param p_oAttributes Attributes passed from parser.  This may be needed
   * when overriding this function.  Basic species-specific values are
   * already handled by this function.
   * @return true if the value was set successfully; false if the value could
   * not be found.  (This would not be an error, because I need a way to
   * cycle through the objects until one of the objects comes up with a
   * match.)  If a match to a data object is made via XML tag, but the found
   * object is not a ModelVector, this returns false.
   * @throws ModelException if the value could not be assigned to the data
   * object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 7, 2005: Created (LEM)
   */
  public boolean SetVectorValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        java.util.Vector p_oData,
                                        String[] p_sChildXMLTags,
                                        boolean[] p_bAppliesTo,
                                        Attributes oParentAttributes,
                                        Attributes[] p_oAttributes) throws
      ModelException {

    if (sXMLParentTag.equals("gliLight") == false &&
        sXMLParentTag.equals("sailLight") == false &&
        sXMLParentTag.equals("quadratGliLight") == false &&
        sXMLParentTag.equals("gliMap") == false &&
        sXMLParentTag.equals("gliPoint") == false &&
        sXMLParentTag.equals("lightOther") == false &&
        sXMLParentTag.equals("ma_v") == false &&
        sXMLParentTag.equals("pkg") == false &&
        sXMLParentTag.equals("ma_intCodes") == false &&
        sXMLParentTag.equals("ma_floatCodes") == false &&
        sXMLParentTag.equals("ma_charCodes") == false &&
        sXMLParentTag.equals("ma_boolCodes") == false &&
        sXMLParentTag.equals("ma_packageIntCodes") == false &&
        sXMLParentTag.equals("ma_packageFloatCodes") == false &&
        sXMLParentTag.equals("ma_packageCharCodes") == false &&
        sXMLParentTag.equals("ma_packageBoolCodes") == false &&
        sXMLParentTag.equals("grid") == false) {
      return false;
    }

    return super.SetVectorValueByXMLTag(sXMLTag, sXMLParentTag, p_oData,
                                        p_sChildXMLTags, p_bAppliesTo,
                                        oParentAttributes, p_oAttributes);
  }

  /**
   * Writes the settings in XML to a parameter file.  Overridden because light
   * is special.
   * @param jOut File to write to.
   * @param oPop Tree population.
   * @throws ModelException If the file is bad.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>January 10, 2005:  Added GLI map settings (LEM)
   * <br>May 11, 2005: Added special stuff for GLI points (LEM)
   */
  public void WriteXML(BufferedWriter jOut, TreePopulation oPop) throws
      ModelException {

    try {
      Behavior oGLI = GetBehaviorByKey("gli"),
          oQuadrat = GetBehaviorByKey("quadrat"),
          oMap = GetBehaviorByKey("map"),
          oPoints = GetBehaviorByKey("points"),
          oSailLight = GetBehaviorByKey("saillight");
      String sStartTag;

      if (!AnyBehaviorsEnabled()) {
        return;
      }

      //Do we write out quadrat light, gli light, map, or all?
      if (oQuadrat.m_bIsEnabled || oGLI.m_bIsEnabled || oMap.m_bIsEnabled ||
          oPoints.m_bIsEnabled) {

        //We have SOME form of GLI writing to do - figure out what our
        //starting tag should be
        if (oQuadrat.m_bIsEnabled && !oGLI.m_bIsEnabled && !oPoints.m_bIsEnabled) {
          sStartTag = "quadratGliLight";
        }
        else if (oMap.m_bIsEnabled && !oGLI.m_bIsEnabled && !oPoints.m_bIsEnabled) {
          sStartTag = "gliMap";
        }
        else if (oPoints.m_bIsEnabled && !oGLI.m_bIsEnabled && !oMap.m_bIsEnabled) {
          sStartTag = "gliPoint";
        }
        else {
          sStartTag = "gliLight";
        }

        jOut.write("<" + sStartTag + ">");

        //Height of fisheye photo
        if (IsThisDataUsed(m_iHeightOfFishEyePhoto)) {
          WriteData(jOut, m_iHeightOfFishEyePhoto);
        }

        //Beam fraction of global radiation
        WriteData(jOut, m_fBeamFractionOfGlobalRadiation);

        //Clear sky transmission coefficient
        WriteData(jOut, m_fClearSkyTransmissionCoefficient);

        //Julian day growth starts
        WriteData(jOut, m_iJulianDayGrowthStarts);

        //Julian day growth ends
        WriteData(jOut, m_iJulianDayGrowthEnds);

        if (oGLI.m_bIsEnabled) {

          //Minimum sun angle
          WriteData(jOut, m_fGLIMinSunAngle);

          //Number altitude grids
          WriteData(jOut, m_iNumGLIAltDiv);

          //Number azimuth grids
          WriteData(jOut, m_iNumGLIAziDiv);

        }
        else if (oQuadrat.m_bIsEnabled) {

          //Minimum sun angle
          WriteData(jOut, m_fQuadratMinSunAngle);

          //Number altitude grids
          WriteData(jOut, m_iNumQuadratAltDiv);

          //Number azimuth grids
          WriteData(jOut, m_iNumQuadratAziDiv);

          //Quadrat light height
          WriteData(jOut, m_fQuadratLightHeight);

        }
        else if (oMap.m_bIsEnabled) {
          //Minimum sun angle
          WriteData(jOut, m_fMapMinSunAngle);

          //Number altitude grids
          WriteData(jOut, m_iNumMapAltDiv);

          //Number azimuth grids
          WriteData(jOut, m_iNumMapAziDiv);

          //Map light height
          WriteData(jOut, m_fMapLightHeight);
        }
        else if (oPoints.m_bIsEnabled) {
          //This is where the points tag is the only one
          //Minimum sun angle
          WriteData(jOut, m_fPointsMinSunAngle);

          //Number altitude grids
          WriteData(jOut, m_iNumPointsAltDiv);

          //Number azimuth grids
          WriteData(jOut, m_iNumPointsAziDiv);

          //Output filename
          WriteData(jOut, m_sGLIPointsOutFile);
        }

        //Light transmission coefficient - make sure to write all species
        WriteSpeciesSpecificValue(jOut, mp_fLightTransmissionCoefficient, oPop);

        //Snag light transmission coefficients
        WriteSpeciesSpecificValue(jOut, mp_fSnagClass1LightTransmissionCoefficient, oPop);
        WriteSpeciesSpecificValue(jOut, mp_fSnagClass2LightTransmissionCoefficient, oPop);
        WriteSpeciesSpecificValue(jOut, mp_fSnagClass3LightTransmissionCoefficient, oPop);

        //Snag size classes
        WriteData(jOut, m_iSnagAgeClass1);
        WriteData(jOut, m_iSnagAgeClass2);

        //Now - very last - if applicable - write GLI points
        //Otherwise our parsing gets messed up
        if (oPoints.m_bIsEnabled) {
            //GLI points
           for (int i = 0; i < mp_oPoints.size(); i++) {
             ((Points) mp_oPoints.elementAt(i)).WriteXML(jOut);
           }
        }

        jOut.write("</" + sStartTag + ">");

        //If quadrat's enabled, write its info
        if (oQuadrat.m_bIsEnabled && sStartTag.equals("quadratGliLight") == false) {
          jOut.write("<quadratGliLight>");
          WriteData(jOut, m_fQuadratMinSunAngle);
          WriteData(jOut, m_iNumQuadratAltDiv);
          WriteData(jOut, m_iNumQuadratAziDiv);
          WriteData(jOut, m_fQuadratLightHeight);
          jOut.write("</quadratGliLight>");
        }


        //If there was the map creator, write its info
        if (oMap.m_bIsEnabled && sStartTag.equals("gliMap") == false) {
          jOut.write("<gliMap>");
          WriteData(jOut, m_fMapMinSunAngle);
          WriteData(jOut, m_iNumMapAltDiv);
          WriteData(jOut, m_iNumMapAziDiv);
          WriteData(jOut, m_fMapLightHeight);
          jOut.write("</gliMap>");
        }

        //If there was the points file, write its info
        if (oPoints.m_bIsEnabled && sStartTag.equals("gliPoint") == false) {
          jOut.write("<gliPoint>");
          WriteData(jOut, m_fPointsMinSunAngle);
          WriteData(jOut, m_iNumPointsAltDiv);
          WriteData(jOut, m_iNumPointsAziDiv);
          WriteData(jOut, m_sGLIPointsOutFile);
          for (int i = 0; i < mp_oPoints.size(); i++) {
            ((Points) mp_oPoints.elementAt(i)).WriteXML(jOut);
          }
          jOut.write("</gliPoint>");
        }
      }

      //Now write sail light, if enabled
      if (oSailLight.m_bIsEnabled) {

        jOut.write("<sailLight>");

        //If there isn't GLI or quadrat, write here the basic stuff
        if (!oGLI.m_bIsEnabled && !oGLI.m_bIsEnabled && !oMap.m_bIsEnabled) {

          //Height of fisheye photo
          WriteData(jOut, m_iHeightOfFishEyePhoto);

          //Beam fraction of global radiation
          WriteData(jOut, m_fBeamFractionOfGlobalRadiation);

          //Clear sky transmission coefficient
          WriteData(jOut, m_fClearSkyTransmissionCoefficient);

          //Julian day growth starts
          WriteData(jOut, m_iJulianDayGrowthStarts);

          //Julian day growth ends
          WriteData(jOut, m_iJulianDayGrowthEnds);

          //Light transmission coefficient - make sure to write all species
          WriteSpeciesSpecificValue(jOut, mp_fLightTransmissionCoefficient, oPop);

          //Snag light transmission coefficients
          WriteSpeciesSpecificValue(jOut, mp_fSnagClass1LightTransmissionCoefficient, oPop);
          WriteSpeciesSpecificValue(jOut, mp_fSnagClass2LightTransmissionCoefficient, oPop);
          WriteSpeciesSpecificValue(jOut, mp_fSnagClass3LightTransmissionCoefficient, oPop);

          //Snag size classes
          WriteData(jOut, m_iSnagAgeClass1);
          WriteData(jOut, m_iSnagAgeClass2);
        }

        WriteData(jOut, m_fSailLightMaxShadingRadius);

        WriteData(jOut, m_fSailLightMaskAngle);

        //Crown fraction of height
        WriteData(jOut, m_iCrownFractionOfHeight);

        jOut.write("</sailLight>");
      }

      //Write out the other light behaviors, if enabled
      if (GetBehaviorByKey("light filter").m_bIsEnabled || GetBehaviorByKey("storm light").m_bIsEnabled ||
          GetBehaviorByKey("basal area light").m_bIsEnabled) {

        jOut.write("<lightOther>");
        if (GetBehaviorByKey("light filter").m_bIsEnabled) {
          WriteData(jOut, m_fFilterLightTransmissionCoefficient);
          WriteData(jOut, m_fFilterHeight);
        }

        if (GetBehaviorByKey("storm light").m_bIsEnabled) {
          WriteData(jOut, m_fStmLightIntercept);
          WriteData(jOut, m_fStmLightMaxRadius);
          WriteData(jOut, m_fStmLightMinCanopyTrees);
          WriteData(jOut, m_fStmLightRandPar);
          WriteData(jOut, m_fStmLightSlope);
          WriteData(jOut, m_iStmLightMaxDmgTime);
          WriteData(jOut, m_iStmLightMaxSnagDmgTime);
          WriteData(jOut, m_iStmLightStochasticity);
        }

        if (GetBehaviorByKey("basal area light").m_bIsEnabled) {
          WriteData(jOut, m_fBasalAreaLightA);
          WriteData(jOut, m_fBasalAreaLightAngiospermB);
          WriteData(jOut, m_fBasalAreaLightAngiospermC);
          WriteData(jOut, m_fBasalAreaLightConiferB);
          WriteData(jOut, m_fBasalAreaLightConiferC);
          WriteData(jOut, m_fBasalAreaLightChangeThreshold);
          WriteData(jOut, m_fBasalAreaLightMinDBH);
          WriteData(jOut, m_fBasalAreaLightSigma);
          WriteSpeciesSpecificValue(jOut, mp_iBasalAreaLightWhatType, oPop);
        }
        jOut.write("</lightOther>");

      }

    }
    catch (java.io.IOException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_FILE;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = "There was a problem writing the parameter file.";
      throw (oErr);
    }
  }

  /**
   * Validates the data prior to its use in, for instance, a parameter file.
   * @param oPop TreePopulation object
   * @throws ModelException in any of the following cases:
   * <ul>
   * <li>If GLI,quadrat light, or map creator is used, and any of the following
   * are not greater than zero:
   * <ul>
   * <li>m_iNumGLIAltDiv</li>
   * <li>m_iNumGLIAziDiv</li>
   * <li>m_iNumQuadratAltDiv</li>
   * <li>m_iNumQuadratAziDiv</li>
   * <li>m_iNumMapAltDiv</li>
   * <li>m_iNumMapAziDiv</li>
   * <li>m_fGLIMinSunAngle</li>
   * <li>m_fQuadratMinSunAngle</li>
   * <li>m_fMapMinSunAngle</li>
   * <li>
   * </ul></li>
   * <li>m_fClearSkyTransmissionCoefficient is 0.</li>
   * <li>mp_fLightTransmissionCoefficient for any species is not between 0
   * and 1.</li>
   * <li>If m_fBeamFractionOfGlobalRadiation is not a valid proportion.</li>
   * <li>If either m_iJulianDayGrowthStarts or m_iJulianDayGrowthEnds is not
   * between 1 and 365 (inclusive).</li>
   * <li>If quadrat light is used and m_fQuadratLightHeight is negative.</li>
   * <li>If GLI map is used and m_fMapLightHeight is negative.</li>
   * <li>If the number of years per timestep is not an integer value.</li>
   * <li>If sail light is used, and m_fSailLightMaxShadingRadius is not greater
   * than zero.</li>
   * <li>If light filter is enabled, and either the filter height or the
   * filter light transmission coefficient is negative, or the filter height is
   * zero.</li>
   * <li>If the run has snags in it and the values for the snag light
   * transmission coefficients aren't proportions, or the snag age classes
   * aren't positive numbers.</li>
   * <li>If Storm Light is enabled:
   * <li>Max years to affect light cannot be a negative number.</li>
   * <li>Min canopy trees cannot be a negative number.</li></li>
   * <li>If GLI points is enabled, points must be furnished in a file (or in
   * a previously loaded XML file) and must be valid</li>
   * <li>If Basal Area Light is enabled:
   * <li>The value for either c cannot equal 0.</li>
   * <li>The basal area change threshold cannot be less than 0.</li>
   * <li>The minimum DBH cannot be less than 0.</li>
   * <li>Grid cell lengths of "Basal Area Light" must divide evenly by the plot
   * lengths.</li></li>
   * </ul>
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>January 10, 2005:  Added GLI Map Creator parameters (LEM)
   * <br>December 8, 2005:  Added Density Light parameters (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    Behavior oGLI = GetBehaviorByKey("gli"),
        oQuadrat = GetBehaviorByKey("quadrat"),
        oMap = GetBehaviorByKey("map"),
        oSailLight = GetBehaviorByKey("saillight"),
        oPoints = GetBehaviorByKey("points");

    if (GetBehaviorByKey("light filter").m_bIsEnabled) {
      MakeSurePositive(m_fFilterHeight);
      MakeSureNonNegative(m_fFilterLightTransmissionCoefficient);
    }

    if (GetBehaviorByKey("storm light").m_bIsEnabled) {
      MakeSureNonNegative(m_iStmLightMaxDmgTime);
      MakeSureNonNegative(m_fStmLightMinCanopyTrees);
    }

    if (GetBehaviorByKey("basal area light").m_bIsEnabled) {
      MakeSureNonZero(m_fBasalAreaLightAngiospermC);
      MakeSureNonZero(m_fBasalAreaLightConiferC);
      MakeSureNonNegative(m_fBasalAreaLightMinDBH);
      MakeSureNonNegative(m_fBasalAreaLightChangeThreshold);
      //Make sure the grid cell lengths of the basal light grid divide
      //evenly into the plot lengths
      int i;
      for (i = 0; i < mp_oGrids.length; i++) {
        if (mp_oGrids[i].GetName().equals("Basal Area Light")) {
          Plot oPlot = m_oManager.GetPlot();
          if (oPlot.GetPlotXLength() % mp_oGrids[i].GetXCellLength() > 0.001 ||
              oPlot.GetPlotYLength() % mp_oGrids[i].GetYCellLength() > 0.001)
            throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The grid cell lengths for the " +
                               mp_oGrids[i].GetName() + " grid must divide evenly into the plot lengths."));
          break;
        }
      }
    }

    //First make sure some behavior that takes parameters is enabled -
    //otherwise don't validate
    if (!oGLI.m_bIsEnabled && !oQuadrat.m_bIsEnabled && !oMap.m_bIsEnabled &&
        !oSailLight.m_bIsEnabled && !oPoints.m_bIsEnabled) {
      return;
    }

    MakeSureRightSize(mp_fLightTransmissionCoefficient, oPop.GetNumberOfSpecies());
    MakeSureAllAreProportions(mp_fLightTransmissionCoefficient);
    MakeSureIsProportion(m_fBeamFractionOfGlobalRadiation);
    MakeSurePositive(m_fClearSkyTransmissionCoefficient);

    //If there are any snags enabled anywhere, make sure that the snag data is
    //filled out
    if (m_oManager.GetSnagAwareness()) {
      MakeSureAllAreProportions(mp_fSnagClass1LightTransmissionCoefficient);
      MakeSureAllAreProportions(mp_fSnagClass2LightTransmissionCoefficient);
      MakeSureAllAreProportions(mp_fSnagClass3LightTransmissionCoefficient);
      MakeSurePositive(m_iSnagAgeClass1);
      MakeSurePositive(m_iSnagAgeClass2);
      if (m_iSnagAgeClass2.GetValue() <= m_iSnagAgeClass1.GetValue()) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 m_iSnagAgeClass2.GetDescriptor() + " must be greater than " +
                                 m_iSnagAgeClass1.GetDescriptor());
      }
    }

    //Make sure the growing season days are proper julian dates
    if (m_iJulianDayGrowthStarts.GetValue() < 1 ||
        m_iJulianDayGrowthStarts.GetValue() > 365 ||
        m_iJulianDayGrowthEnds.GetValue() < 1 ||
        m_iJulianDayGrowthEnds.GetValue() > 365) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = m_iJulianDayGrowthStarts.GetDescriptor() + " and " +
          m_iJulianDayGrowthEnds.GetDescriptor() +
          " must have values between 1 and 365.";
      throw (oErr);
    }

    //Validate the sky grid data for any GLI behavior that's enabled
    if ( oGLI.m_bIsEnabled) {
      MakeSurePositive(m_iNumGLIAltDiv);
      MakeSurePositive(m_iNumGLIAziDiv);
      MakeSurePositive(m_fGLIMinSunAngle);
    }

    if (oQuadrat.m_bIsEnabled) {
      MakeSurePositive(m_iNumQuadratAltDiv);
      MakeSurePositive(m_iNumQuadratAziDiv);
      MakeSurePositive(m_fQuadratMinSunAngle);
      MakeSureNonNegative(m_fQuadratLightHeight);
    }
    if (oMap.m_bIsEnabled) {
      MakeSurePositive(m_iNumMapAltDiv);
      MakeSurePositive(m_iNumMapAziDiv);
      MakeSurePositive(m_fMapMinSunAngle);
      MakeSureNonNegative(m_fMapLightHeight);
    }

    if (null != oSailLight && oSailLight.m_bIsEnabled) {
      MakeSurePositive(m_fSailLightMaxShadingRadius);
    }

    float fNumberYearsPerTimestep = m_oManager.GetPlot().
        GetNumberOfYearsPerTimestep();
    if (java.lang.Math.abs(fNumberYearsPerTimestep -
                           (int) fNumberYearsPerTimestep) > 0) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = "Light behaviors require that the number of years per timestep be a whole number.";
      throw (oErr);
    }

    if (oPoints.m_bIsEnabled) {
      MakeSurePositive(m_iNumPointsAltDiv);
      MakeSurePositive(m_iNumPointsAziDiv);
      MakeSurePositive(m_fPointsMinSunAngle);

      //If we have no input filename and no existing points, squawk
      if (m_sGLIPointsInFile.GetValue().length() == 0 && mp_oPoints.size() == 0) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "You must include a GLI points file.");
      }

      //If we have a filename, read it
      if (m_sGLIPointsInFile.GetValue().length() != 0) {
        AddGLIPointsFile(m_sGLIPointsInFile.GetValue());
      }

      //Make sure we have an output file name
      if (m_sGLIPointsOutFile.GetValue().length() == 0) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "You must provide a value for "+m_sGLIPointsOutFile.GetDescriptor());
      }
    }
  }

  /**
   * Accepts an XML parent tag (empty, no data) from the parser.  This function
   * watches for tag li_GLIPoint.
   * @param sXMLTag The XML tag.
   * @param oAttributes The attributes of this object.
   * @throws ModelException if data is missing or invalid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 11, 2005: Submitted in beta version (LEM)
   */
  public void ReadXMLParentTag(String sXMLTag, Attributes oAttributes) throws
      ModelException {
    if (sXMLTag.equals("li_GLIPoint")) {
      Plot oPlot = m_oManager.GetPlot();

      float fX = new Float( (String) oAttributes.getValue("x")).
          floatValue(),
          fY = new Float( (String) oAttributes.getValue("y")).
          floatValue(),
          fHeight = new Float( (String) oAttributes.getValue("h")).floatValue();
      mp_oPoints.add(new Points(fX, fY, fHeight, oPlot));
    }
    super.ReadXMLParentTag(sXMLTag, oAttributes);
  }

  /**
   * Makes sure values go to the correct behavior.  We override this function
   * because some values (number of azimuth angles, number of altitude angles,
   * and minimum sun angle) can apply to any of three behaviors, so we want to
   * make sure they are assigned to the correct one.
   * @param sXMLTag XML tag of object to assign.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Ignored.
   * @param oData Data to assign.
   * @return true if assigned, or false if not.
   * @throws ModelException not thrown here.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>May 11, 2005:  Added GLI points stuff (LEM)
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        Attributes oAttributes,
                                        Object oData) throws
      ModelException {

    //First - check to make sure they could possibly belong to this behavior
    if (sXMLParentTag.equals("gliLight") == false &&
        sXMLParentTag.equals("sailLight") == false &&
        sXMLParentTag.equals("quadratGliLight") == false &&
        sXMLParentTag.equals("gliPoint") == false &&
        sXMLParentTag.equals("lightOther") == false &&
        sXMLParentTag.equals("gliMap") == false &&
        sXMLParentTag.equals("ma_v") == false &&
        sXMLParentTag.equals("pkg") == false &&
        sXMLParentTag.equals("ma_intCodes") == false &&
        sXMLParentTag.equals("ma_floatCodes") == false &&
        sXMLParentTag.equals("ma_charCodes") == false &&
        sXMLParentTag.equals("ma_boolCodes") == false &&
        sXMLParentTag.equals("ma_packageIntCodes") == false &&
        sXMLParentTag.equals("ma_packageFloatCodes") == false &&
        sXMLParentTag.equals("ma_packageCharCodes") == false &&
        sXMLParentTag.equals("ma_packageBoolCodes") == false &&
        sXMLParentTag.equals("grid") == false) {
      return false;
    }

    //There are duplicate values; if this is one, make sure it assigned to
    //the correct variable.
    if (sXMLParentTag.equals("gliLight")) {
      //Number of altitude angles?
      if (sXMLTag.equals(m_iNumGLIAltDiv.GetXMLTag())) {
        SetSingleValue(m_iNumGLIAltDiv, oData);

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_iNumQuadratAltDiv.GetValue() == 0) {
          m_iNumQuadratAltDiv.SetValue(m_iNumGLIAltDiv.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_iNumMapAltDiv.GetValue() == 0) {
          m_iNumMapAltDiv.SetValue(m_iNumGLIAltDiv.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_iNumPointsAltDiv.GetValue() == 0) {
          m_iNumPointsAltDiv.SetValue(m_iNumGLIAltDiv.GetValue());
        }

        return true;
      }

      //Number of azimuth angles?
      if (sXMLTag.equals(m_iNumGLIAziDiv.GetXMLTag())) {
        SetSingleValue(m_iNumGLIAziDiv, oData);

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_iNumQuadratAziDiv.GetValue() == 0) {
          m_iNumQuadratAziDiv.SetValue(m_iNumGLIAziDiv.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_iNumMapAziDiv.GetValue() == 0) {
          m_iNumMapAziDiv.SetValue(m_iNumGLIAziDiv.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_iNumPointsAziDiv.GetValue() == 0) {
          m_iNumPointsAziDiv.SetValue(m_iNumGLIAziDiv.GetValue());
        }

        return true;
      }

      //Sun angle?
      if (sXMLTag.equals(m_fGLIMinSunAngle.GetXMLTag())) {
        SetSingleValue(m_fGLIMinSunAngle, oData);

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_fQuadratMinSunAngle.GetValue() == 0) {
          m_fQuadratMinSunAngle.SetValue(m_fGLIMinSunAngle.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_fMapMinSunAngle.GetValue() == 0) {
          m_fMapMinSunAngle.SetValue(m_fGLIMinSunAngle.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_fPointsMinSunAngle.GetValue() == 0) {
          m_fPointsMinSunAngle.SetValue(m_fGLIMinSunAngle.GetValue());
        }

        return true;
      }

    }
    else if (sXMLParentTag.equals("quadratGliLight")) {
      //Number of altitude angles?
      if (sXMLTag.equals(m_iNumQuadratAltDiv.GetXMLTag())) {
        SetSingleValue(m_iNumQuadratAltDiv, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_iNumGLIAltDiv.GetValue() == 0) {
          m_iNumGLIAltDiv.SetValue(m_iNumQuadratAltDiv.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_iNumMapAltDiv.GetValue() == 0) {
          m_iNumMapAltDiv.SetValue(m_iNumQuadratAltDiv.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_iNumPointsAltDiv.GetValue() == 0) {
          m_iNumPointsAltDiv.SetValue(m_iNumQuadratAltDiv.GetValue());
        }

        return true;
      }

      //Number of azimuth angles?
      if (sXMLTag.equals(m_iNumQuadratAziDiv.GetXMLTag())) {
        SetSingleValue(m_iNumQuadratAziDiv, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_iNumGLIAziDiv.GetValue() == 0) {
          m_iNumGLIAziDiv.SetValue(m_iNumQuadratAziDiv.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_iNumMapAziDiv.GetValue() == 0) {
          m_iNumMapAziDiv.SetValue(m_iNumQuadratAziDiv.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_iNumPointsAziDiv.GetValue() == 0) {
          m_iNumPointsAziDiv.SetValue(m_iNumQuadratAziDiv.GetValue());
        }

        return true;
      }

      //Sun angle?
      if (sXMLTag.equals(m_fQuadratMinSunAngle.GetXMLTag())) {
        SetSingleValue(m_fQuadratMinSunAngle, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_fGLIMinSunAngle.GetValue() == 0) {
          m_fGLIMinSunAngle.SetValue(m_fQuadratMinSunAngle.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_fMapMinSunAngle.GetValue() == 0) {
          m_fMapMinSunAngle.SetValue(m_fQuadratMinSunAngle.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_fPointsMinSunAngle.GetValue() == 0) {
          m_fPointsMinSunAngle.SetValue(m_fQuadratMinSunAngle.GetValue());
        }

        return true;
      }
    }
    else if (sXMLParentTag.equals("gliMap")) {
      //Number of altitude angles?
      if (sXMLTag.equals(m_iNumMapAltDiv.GetXMLTag())) {
        SetSingleValue(m_iNumMapAltDiv, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_iNumGLIAltDiv.GetValue() == 0) {
          m_iNumGLIAltDiv.SetValue(m_iNumMapAltDiv.GetValue());
        }

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_iNumQuadratAltDiv.GetValue() == 0) {
          m_iNumQuadratAltDiv.SetValue(m_iNumMapAltDiv.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_iNumPointsAltDiv.GetValue() == 0) {
          m_iNumPointsAltDiv.SetValue(m_iNumMapAltDiv.GetValue());
        }

        return true;
      }

      //Number of azimuth angles?
      if (sXMLTag.equals(m_iNumMapAziDiv.GetXMLTag())) {
        SetSingleValue(m_iNumMapAziDiv, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_iNumGLIAziDiv.GetValue() == 0) {
          m_iNumGLIAziDiv.SetValue(m_iNumMapAziDiv.GetValue());
        }

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_iNumQuadratAziDiv.GetValue() == 0) {
          m_iNumQuadratAziDiv.SetValue(m_iNumMapAziDiv.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_iNumPointsAziDiv.GetValue() == 0) {
          m_iNumPointsAziDiv.SetValue(m_iNumMapAziDiv.GetValue());
        }

        return true;
      }

      //Sun angle?
      if (sXMLTag.equals(m_fMapMinSunAngle.GetXMLTag())) {
        SetSingleValue(m_fMapMinSunAngle, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_fGLIMinSunAngle.GetValue() == 0) {
          m_fGLIMinSunAngle.SetValue(m_fMapMinSunAngle.GetValue());
        }

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_fQuadratMinSunAngle.GetValue() == 0) {
          m_fQuadratMinSunAngle.SetValue(m_fMapMinSunAngle.GetValue());
        }

        //Cross-set with gli points, if it hasn't been set yet
        if (m_fPointsMinSunAngle.GetValue() == 0) {
          m_fPointsMinSunAngle.SetValue(m_fMapMinSunAngle.GetValue());
        }

        return true;
      }
    }

    else if (sXMLParentTag.equals("gliPoint")) {
      //Number of altitude angles?
      if (sXMLTag.equals(m_iNumPointsAltDiv.GetXMLTag())) {
        SetSingleValue(m_iNumPointsAltDiv, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_iNumGLIAltDiv.GetValue() == 0) {
          m_iNumGLIAltDiv.SetValue(m_iNumPointsAltDiv.GetValue());
        }

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_iNumQuadratAltDiv.GetValue() == 0) {
          m_iNumQuadratAltDiv.SetValue(m_iNumPointsAltDiv.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_iNumMapAltDiv.GetValue() == 0) {
          m_iNumMapAltDiv.SetValue(m_iNumPointsAltDiv.GetValue());
        }

        return true;
      }

      //Number of azimuth angles?
      if (sXMLTag.equals(m_iNumPointsAziDiv.GetXMLTag())) {
        SetSingleValue(m_iNumPointsAziDiv, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_iNumGLIAziDiv.GetValue() == 0) {
          m_iNumGLIAziDiv.SetValue(m_iNumPointsAziDiv.GetValue());
        }

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_iNumQuadratAziDiv.GetValue() == 0) {
          m_iNumQuadratAziDiv.SetValue(m_iNumPointsAziDiv.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_iNumMapAziDiv.GetValue() == 0) {
          m_iNumMapAziDiv.SetValue(m_iNumPointsAziDiv.GetValue());
        }

        return true;
      }

      //Sun angle?
      if (sXMLTag.equals(m_fPointsMinSunAngle.GetXMLTag())) {
        SetSingleValue(m_fPointsMinSunAngle, oData);

        //Cross-set with gli light, if it hasn't been set yet
        if (m_fGLIMinSunAngle.GetValue() == 0) {
          m_fGLIMinSunAngle.SetValue(m_fPointsMinSunAngle.GetValue());
        }

        //Cross-set with quadrat, if it hasn't been set yet
        if (m_fQuadratMinSunAngle.GetValue() == 0) {
          m_fQuadratMinSunAngle.SetValue(m_fPointsMinSunAngle.GetValue());
        }

        //Cross-set with gli map, if it hasn't been set yet
        if (m_fMapMinSunAngle.GetValue() == 0) {
          m_fMapMinSunAngle.SetValue(m_fPointsMinSunAngle.GetValue());
        }

        return true;
      }
    }
      return super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag,
                                     oAttributes, oData);
  }
}

/** For listing points for GLI points calculations */
class Points {
  private float m_fX; /**<X coordinate*/
  private float m_fY; /**<Y coordinate*/
  private float m_fHeight; /**<Height in m*/

  /**
   * Constructor.
   * @param fX float X coordinate
   * @param fY float Y coordinate
   * @param fHeight float Height in meters
   * @param oPlot Plot Plot object
   * @throws ModelException if an X or Y coordinate is outside the plot,
   * or if the height is negative.
   */
  public Points(float fX, float fY, float fHeight, Plot oPlot) throws ModelException {
    if (fX < 0 || fX >= oPlot.GetPlotXLength() || fY < 0 || fY >= oPlot.GetPlotYLength()) {
      throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "There was a GLI point value outside of the plot.  X = \"" + fX + "\", Y = \"" + fY + "\".");
    }

    if (fHeight < 0) {
      throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "There was a negative height in a GLI point. Height = \""+fHeight+"\".");
    }
    m_fX = fX;
    m_fY = fY;
    m_fHeight = fHeight;
  }

  /**
   * Writes the XML for this point.
   * @param jOut BufferedWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  public void WriteXML(BufferedWriter jOut) throws java.io.IOException {
    jOut.write("<li_GLIPoint x=\""+m_fX+"\" y=\""+m_fY+"\" h=\""+m_fHeight+"\"/>");
  }

  public float GetX() {return m_fX;};
  public float GetY() {return m_fY;};
  public float GetHeight() {return m_fHeight;};
}
