package javawrapper;

/**
 * This class organizes the germination behavior.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class SeedPredationBehaviors
    extends BehaviorTypeBase {

  /**Predator initial density in number per square meter*/
  private ModelFloat m_fPredatorInitialDensity = new ModelFloat(0,
      "Predator Initial Density - num/sq m", "pr_predatorInitialDensity");
  /**Max instantaneous rate at which predator abundance declines in the
       absense of food, in number of predators per week*/
  private ModelFloat m_fMaxInstantaneousDeclineRate = new ModelFloat(0,
      "Max Decline Rate in Absence of Food, predators/week",
      "pr_maxInstantaneousDeclineRate");
  /**Predator population's demographic efficiency*/
  private ModelFloat m_fDemographicEfficiency = new ModelFloat(0,
      "Predator Population Demographic Efficiency", "pr_demographicEfficiency");
  /**Density-dependent coefficient*/
  private ModelFloat m_fDensityDependentCoefficient = new ModelFloat(0,
      "Density-Dependent Coefficient", "pr_densityDependentCoefficient");
  /**Maximum intake rate - number of seeds per predator per day*/
  private ModelFloat m_fMaxIntakeRate = new ModelFloat(0,
      "Max Intake Rate - seeds per predator per day", "pr_maxIntakeRate");
  /**Foraging efficiency*/
  private ModelFloat m_fForagingEfficiency = new ModelFloat(0,
      "Foraging Efficiency",
      "pr_foragingEfficiency");
  /**Proportion of seeds, between 0 and 1, germinating each week in the
   * germination period*/
  private ModelFloat m_fProportionGerminating = new ModelFloat(0,
      "Proportion of Seeds Removed Each Week During Germination Period",
      "pr_proportionGerminating");

  /**Number of weeks for which seedfall occurs*/
  private ModelInt m_iNumWeeksSeedFall = new ModelInt(0,
      "Number of Weeks in Which Seedfall Occurs", "pr_numWeeksSeedFall");
  /**Number of weeks to run model - must be less than or equal to 52*/
  private ModelInt m_iNumWeeksToModel = new ModelInt(0,
      "Weeks to Run Seed Predation Model (1 - 52)",
      "pr_numWeeksToModel");
  /**Week number in which germination begins*/
  private ModelInt m_iWeekGerminationStarts = new ModelInt(0,
      "Week During Predation Model Run Germination Begins",
      "pr_weekGerminationStarts");

  /**Output filename*/
  private ModelString m_sOutputFilename = new ModelString("",
      "Seed Predation Output Filename, If Desired", "pr_outputFilename");

  /**Whether or not to preserve predator densities between model timesteps
   * (true) or re-initialize with predator initial density (false)*/
  private ModelEnum m_iPreservePredatorDensities = new ModelEnum(new int[] {0,
      1}
      ,
      new String[] {"false", "true"}
      ,
      "Preserve Predator Densities Between SORTIE Timesteps",
      "pr_preservePredatorDensities");

  /**
   * Constructor.  Sets up the child behavior list and adds data appropriately.
   * @param oManager GUIManager object.
   * @throws ModelException passed through from called methods.  Should never
   * be thrown.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public SeedPredationBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Seed predation", "funcResponsePredation");

    m_sHelpID = "data.seed_predation_behaviors";

    //Set up the functional response predation behavior
    mp_oChildBehaviors = new Behavior[1];

    mp_oChildBehaviors[0] = new Behavior("funcResponsePredation",
                                         "Functional response seed predation",
                                         "functional response seed predation",
                                         (float)1);
    //Disallow for all but seeds
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEED, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.ADULT, false);

    //Add all data
    mp_oChildBehaviors[0].AddRequiredData(m_fPredatorInitialDensity);
    mp_oChildBehaviors[0].AddRequiredData(m_fMaxInstantaneousDeclineRate);
    mp_oChildBehaviors[0].AddRequiredData(m_fDemographicEfficiency);
    mp_oChildBehaviors[0].AddRequiredData(m_fDensityDependentCoefficient);
    mp_oChildBehaviors[0].AddRequiredData(m_fMaxIntakeRate);
    mp_oChildBehaviors[0].AddRequiredData(m_fForagingEfficiency);
    mp_oChildBehaviors[0].AddRequiredData(m_iNumWeeksSeedFall);
    mp_oChildBehaviors[0].AddRequiredData(m_iNumWeeksToModel);
    mp_oChildBehaviors[0].AddRequiredData(m_iWeekGerminationStarts);
    mp_oChildBehaviors[0].AddRequiredData(m_iPreservePredatorDensities);
    mp_oChildBehaviors[0].AddRequiredData(m_fProportionGerminating);
    mp_oChildBehaviors[0].AddRequiredData(m_sOutputFilename);

    //Now add proportion germinating to the all data list
    mp_oAllData.add(m_fPredatorInitialDensity);
    mp_oAllData.add(m_fMaxInstantaneousDeclineRate);
    mp_oAllData.add(m_fDemographicEfficiency);
    mp_oAllData.add(m_fDensityDependentCoefficient);
    mp_oAllData.add(m_fMaxIntakeRate);
    mp_oAllData.add(m_fForagingEfficiency);
    mp_oAllData.add(m_iNumWeeksSeedFall);
    mp_oAllData.add(m_iNumWeeksToModel);
    mp_oAllData.add(m_iWeekGerminationStarts);
    mp_oAllData.add(m_iPreservePredatorDensities);
    mp_oAllData.add(m_fProportionGerminating);
    mp_oAllData.add(m_sOutputFilename);

    //Make the one grid
    mp_oGrids = new Grid[1];

    //Create the data member
    DataMember[] p_oDataMembers = new DataMember[1];

    p_oDataMembers[0] = new DataMember("Number of predators", "num preds",
                                       DataMember.FLOAT);

    mp_oGrids[0] = new Grid("Seed Predators", p_oDataMembers, null, 8,
                            8);

    //Add to the substrate behavior
    GetBehaviorByKey("funcResponsePredation").AddGrid(mp_oGrids[0]);

  }

  /**
   * Does nothing.
   * @param oPop Not needed.
   * @throws javawrapper.ModelException Not thrown.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws javawrapper.ModelException {}

  /**
   * Validates the dataset if seed predation is enabled.
   * @param oPop TreePopulation object.
   * @throws javawrapper.ModelException if:
   * <ul>
   * <li>Number of weeks of seedfall is less than or equal to zero
   * <li>Initial predator density is less than or equal to zero
   * <li>Number of weeks to run the model is less than 0 or greater than 52
   * <li>Week to start germination is less than 0 or greater than 52
   * <li>Proportion germinating is not a proportion
   * </ul>
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws javawrapper.
      ModelException {

    if (!AnyBehaviorsEnabled()) {
      return;
    }

    MakeSurePositive(m_iNumWeeksSeedFall);
    MakeSurePositive(m_fPredatorInitialDensity);
    MakeSurePositive(m_iNumWeeksToModel);
    MakeSureNonNegative(m_iWeekGerminationStarts);
    MakeSureIsProportion(m_fProportionGerminating);

    if (m_iNumWeeksToModel.GetValue() > 52) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = m_iNumWeeksToModel.GetDescriptor() +
          " must not be greater than 52.";
      throw (oErr);
    }

    if (m_iWeekGerminationStarts.GetValue() > 52) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_DATA;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = m_iNumWeeksToModel.GetDescriptor() +
          " must not be greater than 52.";
      throw (oErr);
    }
  }
}
