package javawrapper;

import junit.framework.*;
import java.util.Vector;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the MortalityBehaviors class.
 * Copyright: Copyright (c) Charles D. Canham 2003</p>
 * Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestMortalityBehaviors
    extends TestCase {
  private TreePopulation m_oPop = null;

  public TestMortalityBehaviors(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
    try {
      //Create our tree population
      m_oPop = new TreePopulation(null);
      m_oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});
    }
    catch (ModelException oErr) {
      throw new Exception(oErr.sMessage);
    }
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public void testFormatDataForDisplay() {
    try {
      MortalityBehaviors oMortality = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no mortality behaviors are enabled.
      oMortality = new MortalityBehaviors(null);
      assertEquals(null, oMortality.FormatDataForDisplay(m_oPop));

      //Case:  only GMF mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("gmfmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Light-Dependent Mortality",
          "Mortality at Zero Growth"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only BC mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("bcmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Light-Dependent Mortality",
          "Mortality at Zero Growth"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only adult self-thinning is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("adultselfthin");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Adult Self-Thinning Slope",
          "Maximum DBH for Adult Self-Thinning",
          "Adult Self-Thinning Intercept"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only juvenile self-thinning is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("juvselfthin");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SAPLING,
          m_oPop));
      p_sExpected = new String[] {
          "Juvenile Self-Thinning Slope",
          "Juvenile Self-Thinning Intercept"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only senescence is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("senescence");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Senescence Mortality Alpha",
          "Senescence Mortality Beta",
          "DBH of Maximum Senescence Mortality Rate, as an integer in cm",
          "DBH at Onset of Senescence, in cm"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only adult stochastic mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("adultstochmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Adult Background Mortality Rate"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only juvenile stochastic mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("juvstochmort");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SAPLING,
          m_oPop));
      p_sExpected = new String[] {
          "Juvenile Background Mortality Rate"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only weibull snag mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SNAG,
          m_oPop));
      p_sExpected = new String[] {
          "Weibull Annual \"a\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Upper DBH of Snag Size Class 1",
          "Weibull Upper DBH of Snag Size Class 2"
      };
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only pr nci mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("NCI Mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "NCI Max Radius of Crowding Neighbors, in m",
          "NCI Neighbor DBH Effect (alpha)",
          "NCI Neighbor Distance Effect (beta)",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Max Survival Probability (0-1)",
          "NCI Size Effect Mode, in cm",
          "NCI Size Effect Variance, in cm",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Storm Effect - Medium Damage (0-1)",
          "NCI Storm Effect - Complete Damage (0-1)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Minimum Neighbor DBH, in cm",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "ACRU NCI Lambda Neighbors",
          "ACSA NCI Lambda Neighbors"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only competition mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("competition mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Competition Mortality Shape Parameter (Z)",
          "Competition Mortality Maximum Parameter (max)"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only growth and resource mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("resource mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Growth/Resource - Scaling Factor (rho)",
          "Growth/Resource - Function Mode (mu)",
          "Growth/Resource - Survival Increase with Growth (delta)",
          "Growth/Resource - Low-Growth Survival Parameter (sigma)"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only density self-thinning mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("density self-thinning mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, m_oPop));
      p_sExpected = new String[] {
          "Density Self-Thinning Asymptote (A)",
          "Density Self-Thinning Diameter Effect (C)",
          "Density Self-Thinning Density Effect (S)",
          "Density Self-Thinning Minimum Density for Mortality (#/ha)",
          "Density Self-Thinning Neighborhood Radius, in m"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only logistic bi-level mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("logistic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, m_oPop));
      p_sExpected = new String[] {
          "Logistic Bi-Level - Low-Light \"a\"",
          "Logistic Bi-Level - Low-Light \"b\"",
          "Logistic Bi-Level - High-Light \"a\"",
          "Logistic Bi-Level - High-Light \"b\"",
          "Logistic Bi-Level - High-Light Mortality Threshold (0-100)"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only stochastic bi-level mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("stochastic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, m_oPop));
      p_sExpected = new String[] {
          "Stochastic Bi-Level - Low-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Threshold (0-100)"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only height-gli weibull mortality is enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      oBeh = oMortality.GetBehaviorByKey("height-gli weibull mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, m_oPop));
      p_sExpected = new String[] {
          "Height-GLI Weibull - a",
          "Height-GLI Weibull - b",
          "Height-GLI Weibull - c",
          "Height-GLI Weibull - d",
          "Height-GLI Weibull - Max Mortality (0 - 1)"};
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  all are enabled.
      oMortality = null;
      oMortality = new MortalityBehaviors(null);
      oMortality.DoSetup(m_oPop);
      int i;
      for (i = 0; i < oMortality.mp_oChildBehaviors.length; i++) {
        oMortality.mp_oChildBehaviors[i].m_bIsEnabled = true;
        if (oMortality.mp_oChildBehaviors[i].GetKey().equals("juvstochmort") ||
            oMortality.mp_oChildBehaviors[i].GetKey().equals("juvselfthin") ||
            oMortality.mp_oChildBehaviors[i].GetKey().equals(
            "density self-thinning mortality")) {
          oMortality.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(
              0, TreePopulation.SAPLING, m_oPop));
        }
        else if (oMortality.mp_oChildBehaviors[i].GetKey().equals(
            "weibull snag mortality")) {
          oMortality.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(
              0, TreePopulation.SNAG, m_oPop));
        }
        else {
          oMortality.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(
              0, 3, m_oPop));
        }
      }
      p_sExpected = new String[] {
          "Light-Dependent Mortality",
          "Mortality at Zero Growth",
          "Juvenile Self-Thinning Slope",
          "Juvenile Self-Thinning Intercept",
          "Adult Self-Thinning Slope",
          "Maximum DBH for Adult Self-Thinning",
          "Adult Self-Thinning Intercept",
          "Senescence Mortality Alpha",
          "Senescence Mortality Beta",
          "DBH of Maximum Senescence Mortality Rate, as an integer in cm",
          "DBH at Onset of Senescence, in cm",
          "Adult Background Mortality Rate",
          "Juvenile Background Mortality Rate",
          "Weibull Annual \"a\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"a\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 1 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 2 Mortality",
          "Weibull Annual \"b\" Parameter for Snag Size Class 3 Mortality",
          "Weibull Upper DBH of Snag Size Class 1",
          "Weibull Upper DBH of Snag Size Class 2",
          "Density Self-Thinning Asymptote (A)",
          "Density Self-Thinning Diameter Effect (C)",
          "Density Self-Thinning Density Effect (S)",
          "Density Self-Thinning Minimum Density for Mortality (#/ha)",
          "Density Self-Thinning Neighborhood Radius, in m",
          "NCI Max Radius of Crowding Neighbors, in m",
          "NCI Neighbor DBH Effect (alpha)",
          "NCI Neighbor Distance Effect (beta)",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Max Survival Probability (0-1)",
          "NCI Size Effect Mode, in cm",
          "NCI Size Effect Variance, in cm",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Storm Effect - Medium Damage (0-1)",
          "NCI Storm Effect - Complete Damage (0-1)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Minimum Neighbor DBH, in cm",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "ACRU NCI Lambda Neighbors",
          "ACSA NCI Lambda Neighbors",
          "Growth/Resource - Scaling Factor (rho)",
          "Growth/Resource - Function Mode (mu)",
          "Growth/Resource - Survival Increase with Growth (delta)",
          "Growth/Resource - Low-Growth Survival Parameter (sigma)",
          "Competition Mortality Shape Parameter (Z)",
          "Competition Mortality Maximum Parameter (max)",
          "Logistic Bi-Level - Low-Light \"a\"",
          "Logistic Bi-Level - Low-Light \"b\"",
          "Logistic Bi-Level - High-Light \"a\"",
          "Logistic Bi-Level - High-Light \"b\"",
          "Logistic Bi-Level - High-Light Mortality Threshold (0-100)",
          "Stochastic Bi-Level - Low-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Probability (0-1)",
          "Stochastic Bi-Level - High-Light Mortality Threshold (0-100)",
          "Height-GLI Weibull - a",
          "Height-GLI Weibull - b",
          "Height-GLI Weibull - c",
          "Height-GLI Weibull - d",
          "Height-GLI Weibull - Max Mortality (0 - 1)"
      };
      TestTable(oMortality.FormatDataForDisplay(m_oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for mortality.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }

  }

  /**
   * Tests backwards compatibility for Weibull snag mortality behavior
   * and NCI mortality.
   */
  public void testBackwardsCompatibility() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //***************
       // Weibull snag mortality
       //***************
        //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();
      float fExpected;
      int iNumSpecies = oManager.GetTreePopulation().GetNumberOfSpecies(), i;
      fExpected = 12;
      for (i = 0; i < iNumSpecies; i++) {
        assertEquals(fExpected,
                     ( (Float) oMort.mp_fSnagSizeClass1Dbh.GetValue().
                      elementAt(i)).floatValue(), 0.01);
      }
      fExpected = 30;
      for (i = 0; i < iNumSpecies; i++) {
        assertEquals(fExpected,
                     ( (Float) oMort.mp_fSnagSizeClass2Dbh.GetValue().
                      elementAt(i)).floatValue(), 0.01);
      }
      fExpected = 100;
      assertEquals(100, oMort.m_iDbhAtAsymptoticMaximumMortality.GetValue());
      new File(sFileName).delete();

      //***************
       // NCI mortality
       //***************

    }
    catch (ModelException oErr) {
      fail("Mortality validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

  }

  /**
   * Tests a table set against a list of expected members in the table.  It
   * does not care how many tables are defined in the vector.  It will
   * throw a fail if there are members missing or members that are not
   * expected.  Species are not checked.  Blanks are skipped.
   * @param p_oTablesToTest Vector of tables to test.
   * @param p_sExpected Expected member list.
   */
  private void TestTable(Vector p_oTablesToTest, String[] p_sExpected) {
    Object[][] p_oData;
    Vector p_oOneTable;
    String sMemberName;
    int[] p_iFoundTimes = new int[p_sExpected.length]; //holds the number of
    //times each member was found
    int iNumberOfTables = p_oTablesToTest.size(),
        i, j, k;
    boolean bFound;

    //Initialize the number of times found array to all 0s
    for (i = 0; i < p_iFoundTimes.length; i++) {
      p_iFoundTimes[i] = 0;
    }

    //Analyze each table
    for (i = 0; i < iNumberOfTables; i++) {
      p_oOneTable = (Vector) p_oTablesToTest.elementAt(i);

      //The second element in the vector is the data
      p_oData = (Object[][]) p_oOneTable.elementAt(1);

      //Go through each table row
      for (j = 0; j < p_oData.length; j++) {

        sMemberName = (String) p_oData[j][0];
        bFound = false;

        if (!sMemberName.equals(" ")) {

          //Does it match a member of the expected list?
          for (k = 0; k < p_sExpected.length; k++) {

            if (sMemberName.equals(p_sExpected[k])) {

              bFound = true;
              p_iFoundTimes[k]++;

            }
          }
        }

        //Did we find our member in the expected list?
        if (!bFound) {
          fail(sMemberName + " unexpectedly found in table.");
        }
      }
    }

    //Now look at the number of times we found each expected member - if
    //not 1 for each, fail
    for (i = 0; i < p_iFoundTimes.length; i++) {
      assertEquals("Found " + p_sExpected[i] + " unexpected number of times.",
                   1, p_iFoundTimes[i]);
    }
  }

  /**
   * Tests ValidateData().
   */
  public void testValidateData() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      /********************
        NCI mortality testing
       *********************/
      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Mortality validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 6
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile6();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 6.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 7
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile7();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 7.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 8
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile8();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 8.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 9
    try {
      oManager.ClearCurrentData();
      sFileName = TestNCIMortality.WriteXMLErrorFile9();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 9.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    /********************
      Density self-thinning mortality testing
     *********************/
    try {
    //Valid file 1
      oManager.ClearCurrentData();
      sFileName = TestDensitySelfThinningMortality.WriteValidFile();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Mortality validation failed with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = TestDensitySelfThinningMortality.WriteErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = TestDensitySelfThinningMortality.WriteErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = TestDensitySelfThinningMortality.WriteErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 4
    try {
      oManager.ClearCurrentData();
      sFileName = TestDensitySelfThinningMortality.WriteErrorFile4();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 4.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - error file 5
    try {
      oManager.ClearCurrentData();
      sFileName = TestDensitySelfThinningMortality.WriteErrorFile5();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetMortalityBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in error file 5.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Light threshold not between 0 and 100 for linear bi-level mortality
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("logistic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oMort.mp_fLogBiLevHiLiteThreshold.GetValue().add(new Float(-20));
      oMort.mp_fLogBiLevHiLiteThreshold.GetValue().add(new Float(0));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad light threshold values.");

    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //size class 1 a values = 0 for weibull mortality
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oMort.mp_fSnag1WeibullA.GetValue().add(new Float(0));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad weibull snag \"a\" values.");

    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //size class 2 a values = 0 for weibull mortality
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oMort.mp_fSnag2WeibullA.GetValue().add(new Float(0));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad weibull snag \"a\" values.");

    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }


    //size class 3 a values = 0 for weibull mortality
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oMort.mp_fSnag3WeibullA.GetValue().add(new Float(0));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad weibull snag \"a\" values.");

    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //size class 1 a values < 0 plus non-whole-number b for weibull mortality
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oMort.mp_fSnag1WeibullA.GetValue().add(new Float(-3));
      //Whole number - should be OK
      oMort.mp_fSnag1WeibullB.GetValue().add(new Float(4));
      oMort.ValidateData(oManager.GetTreePopulation());
      //Set to non-whole-number
      oMort.mp_fSnag1WeibullB.GetValue().add(0, new Float(0.25));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad weibull snag \"a\" values.");

    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //size class 2 a values < 0 plus non-whole-number b for weibull mortality
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oMort.mp_fSnag2WeibullA.GetValue().add(new Float(-3));
      //Whole number - should be OK
      oMort.mp_fSnag2WeibullB.GetValue().add(new Float(4));
      oMort.ValidateData(oManager.GetTreePopulation());
      //Set to non-whole-number
      oMort.mp_fSnag2WeibullB.GetValue().add(0, new Float(0.25));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad weibull snag \"a\" values.");

    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //size class 3 a values < 0 plus non-whole-number b for weibull mortality
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("weibull snag mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oMort.mp_fSnag3WeibullA.GetValue().add(new Float(-3));
      //Whole number - should be OK
      oMort.mp_fSnag3WeibullB.GetValue().add(new Float(4));
      oMort.ValidateData(oManager.GetTreePopulation());
      //Set to non-whole-number
      oMort.mp_fSnag3WeibullB.GetValue().add(0, new Float(0.25));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad weibull snag \"a\" values.");

    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    /********************
     Stochastic bi-level mortality testing
    *********************/
   //High-light threshold value not between 0 and 100
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;
      //Should be OK
      oMort.mp_fStochBiLevHiLiteThreshold.GetValue().add(new Float(50));
      oMort.ValidateData(oManager.GetTreePopulation());
      //Set to bad value
      oMort.mp_fStochBiLevHiLiteThreshold.GetValue().add(0, new Float(-20));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad stochastic bi-level threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Low-light mortality probability not between 0 and 1
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;
      //Should be OK
      oMort.mp_fStochBiLevLoLiteMortProb.GetValue().add(new Float(0.5));
      oMort.ValidateData(oManager.GetTreePopulation());
      //Set to bad value
      oMort.mp_fStochBiLevLoLiteMortProb.GetValue().add(0, new Float(2));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad stochastic bi-level low-light probability values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //High-light mortality probability not between 0 and 1
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;
      //Should be OK
      oMort.mp_fStochBiLevHiLiteMortProb.GetValue().add(new Float(0.5));
      oMort.ValidateData(oManager.GetTreePopulation());
      //Set to bad value
      oMort.mp_fStochBiLevHiLiteMortProb.GetValue().add(0, new Float(2));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad stochastic bi-level low-light probability values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Storm light not enabled
   try {
     oManager = new GUIManager(null);

     //Valid file 1
     oManager.ClearCurrentData();
     sFileName = WriteXMLFile2();
     oManager.InputXMLParameterFile(sFileName);
     MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

     Behavior oBeh = oMort.GetBehaviorByKey("stochastic bi-level mortality");
     oBeh.m_bIsEnabled = true;
     oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
     oMort.ValidateData(oManager.GetTreePopulation());
     fail("Mortality validation failed to catch storm light not enabled.");
   }
   catch (ModelException oErr) {
     ;
   }
   catch (java.io.IOException oE) {
     fail("Caught IOException.  Message: " + oE.getMessage());
   }
   finally {
     new File(sFileName).delete();
   }

   /********************
     Height gli weibull mortality testing
    *********************/
   //Max mortality not between 0 and 1
    try {
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      MortalityBehaviors oMort = oManager.GetMortalityBehaviors();

      Behavior oBeh = oMort.GetBehaviorByKey("height-gli weibull mortality");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      //Should be OK
      oMort.mp_fHeightGLIWeibMaxMort.GetValue().add(new Float(0.5));
      oMort.ValidateData(oManager.GetTreePopulation());
      //Set to bad value
      oMort.mp_fHeightGLIWeibMaxMort.GetValue().add(0, new Float(-20));
      oMort.ValidateData(oManager.GetTreePopulation());
      fail("Mortality validation failed to catch bad max mortality values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    System.out.println("Mortality ValidateData testing succeeded.");
  }

  protected String WriteXMLFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>weibull snag mortality</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Snag\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>senescence</behaviorName> ");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>removedead</behaviorName>");
    oOut.write("<version>2.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Snag\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_snagSizeClass1>12</mo_snagSizeClass1>");
    oOut.write("<mo_snagSizeClass2>30</mo_snagSizeClass2>");
    oOut.write("<mo_snag1WeibullA>");
    oOut.write("<mo_s1waVal species=\"Species_3\">0.025</mo_s1waVal>");
    oOut.write("<mo_s1waVal species=\"Species_1\">0.113</mo_s1waVal>");
    oOut.write("</mo_snag1WeibullA>");
    oOut.write("<mo_snag2WeibullA>");
    oOut.write("<mo_s2waVal species=\"Species_3\">0.09</mo_s2waVal>");
    oOut.write("<mo_s2waVal species=\"Species_1\">0.0113</mo_s2waVal>");
    oOut.write("</mo_snag2WeibullA>");
    oOut.write("<mo_snag3WeibullA>");
    oOut.write("<mo_s3waVal species=\"Species_3\">0.0204</mo_s3waVal>");
    oOut.write("<mo_s3waVal species=\"Species_1\">0.1</mo_s3waVal>");
    oOut.write("</mo_snag3WeibullA>");
    oOut.write("<mo_snag1WeibullB>");
    oOut.write("<mo_s1wbVal species=\"Species_3\">3</mo_s1wbVal>");
    oOut.write("<mo_s1wbVal species=\"Species_1\">3.049</mo_s1wbVal>");
    oOut.write("</mo_snag1WeibullB>");
    oOut.write("<mo_snag2WeibullB>");
    oOut.write("<mo_s2wbVal species=\"Species_3\">2</mo_s2wbVal>");
    oOut.write("<mo_s2wbVal species=\"Species_1\">2.2</mo_s2wbVal>");
    oOut.write("</mo_snag2WeibullB>");
    oOut.write("<mo_snag3WeibullB>");
    oOut.write("<mo_s3wbVal species=\"Species_3\">2.5</mo_s3wbVal>");
    oOut.write("<mo_s3wbVal species=\"Species_1\">2.3</mo_s3wbVal>");
    oOut.write("</mo_snag3WeibullB>");
    oOut.write("<mo_randomMortAlpha>");
    oOut.write("<mo_rmaVal species=\"Species_1\">-20.0</mo_rmaVal>");
    oOut.write("<mo_rmaVal species=\"Species_2\">-20.0</mo_rmaVal>");
    oOut.write("<mo_rmaVal species=\"Species_3\">-20.0</mo_rmaVal>");
    oOut.write("</mo_randomMortAlpha>");
    oOut.write("<mo_randomMortBeta>");
    oOut.write("<mo_rmbVal species=\"Species_1\">0.135</mo_rmbVal>");
    oOut.write("<mo_rmbVal species=\"Species_2\">0.135</mo_rmbVal>");
    oOut.write("<mo_rmbVal species=\"Species_3\">0.135</mo_rmbVal>");
    oOut.write("</mo_randomMortBeta>");
    oOut.write("<mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_daoosVal species=\"Species_1\">100.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_2\">100.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_3\">100.0</mo_daoosVal>");
    oOut.write("</mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_dbhAtAsympMaxMort>100</mo_dbhAtAsympMaxMort>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }
}

/**
 * This class is to help group test code.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
class TestDensitySelfThinningMortality {
  /**
   * Writes a file of valid values for density self-thinning.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteValidFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the timestep length is greater than 1.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<rt_timestep>0</rt_timestep>");
    oOut.write("<randomSeed>0</randomSeed>");
    oOut.write("<yearsPerTimestep>3.0</yearsPerTimestep>");
    oOut.write("<plot_lenX>100.0</plot_lenX>");
    oOut.write("<plot_lenY>100.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the neighborhood radius is less than or equal to 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">-9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the minimum density is less than 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">-20</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the self-thinning asymptote is less than or equal
   * to 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">-0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the density effect is less than or equal to 0.
   * @throws IOException if there is a problem writing the file.
   * @return String File to write to.
   */
  public static String WriteErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteCommonStuff(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">-0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes stuff common to all parameter files in this class.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private static void WriteCommonStuff(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<rt_timestep>0</rt_timestep>");
    oOut.write("<randomSeed>0</randomSeed>");
    oOut.write("<yearsPerTimestep>1.0</yearsPerTimestep>");
    oOut.write("<plot_lenX>100.0</plot_lenX>");
    oOut.write("<plot_lenY>100.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }
}

/**
 * This class is to help group test code.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
class TestNCIMortality {

  /**
   * Writes an error condition where Xb for a species = 0.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    oOut.write("<mo_prnciSizeEffectVar>");
    oOut.write("<mo_psevVal species=\"Species_1\">0</mo_psevVal>");
    oOut.write("<mo_psevVal species=\"Species_2\">5</mo_psevVal>");
    oOut.write("</mo_prnciSizeEffectVar>");
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where X0 for a species = 0.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    oOut.write("<mo_prnciSizeEffectMode>");
    oOut.write("<mo_psemVal species=\"Species_1\">0</mo_psemVal>");
    oOut.write("<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>");
    oOut.write("</mo_prnciSizeEffectMode>");
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where med. damage eta for a species is not a
   * proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    oOut.write("<mo_prnciNeighStormEffMedDmg>");
    oOut.write("<mo_pnsemdVal species=\"Species_1\">-0.33</mo_pnsemdVal>");
    oOut.write("<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>");
    oOut.write("</mo_prnciNeighStormEffMedDmg>");
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where full damage eta for a species is not a
   * proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    oOut.write("<mo_prnciNeighStormEffFullDmg>");
    oOut.write("<mo_pnsefdVal species=\"Species_1\">-0.54</mo_pnsefdVal>");
    oOut.write("<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>");
    oOut.write("</mo_prnciNeighStormEffFullDmg>");
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where med. damage storm Effect for a species is
   * not a proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    oOut.write("<mo_prnciStormEffMedDmg>");
    oOut.write("<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>");
    oOut.write("<mo_psemdVal species=\"Species_2\">1.9</mo_psemdVal>");
    oOut.write("</mo_prnciStormEffMedDmg>");
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where full damage storm Effect for a species is
   * not a proportion.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile8() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    oOut.write("<mo_prnciStormEffFullDmg>");
    oOut.write("<mo_psefdVal species=\"Species_1\">-0.42</mo_psefdVal>");
    oOut.write("<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>");
    oOut.write("</mo_prnciStormEffFullDmg>");
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes NCI mortality file with a min neighbor DBH < 0.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  public static String WriteXMLErrorFile9() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    oOut.write("<mo_nciMinNeighborDBH>");
    oOut.write("<mo_nmndVal species=\"Species_1\">10</mo_nmndVal>");
    oOut.write("<mo_nmndVal species=\"Species_2\">-10</mo_nmndVal>");
    oOut.write("</mo_nciMinNeighborDBH>");
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  private static void WriteValidateFilePart1(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  /**
   * Writes a valid NCI mortality file.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>September 6, 2004: Created (LEM)
   */
  public static String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file to test backwards compatibility of NCI mortality.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 16, 2005: Created (LEM)
   */
  public static String WriteXMLCompatFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>puerto rico nci mortality</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where max radius of neighbor Effects is < 0.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    oOut.write("<mo_prnciMaxCrowdingRadius>");
    oOut.write("<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>");
    oOut.write("<mo_pmcrVal species=\"Species_2\">-10</mo_pmcrVal>");
    oOut.write("</mo_prnciMaxCrowdingRadius>");
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error condition where max probability of survival for a species
   * is not between 0 and 1.
   * @return Filename.
   * @throws IOException if there is a problem writing the file.
   */
  public static String WriteXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    WriteBehaviorList(oOut);
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    oOut.write("<mo_prnciMaxPotentialSurvival>");
    oOut.write("<mo_pmpsVal species=\"Species_1\">15.31</mo_pmpsVal>");
    oOut.write("<mo_pmpsVal species=\"Species_2\">-15.31</mo_pmpsVal>");
    oOut.write("</mo_prnciMaxPotentialSurvival>");
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  private static void WriteBehaviorList(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>NCI Mortality</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  private static void WriteNCIMaxCrowdingRadius(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciMaxCrowdingRadius>");
    oOut.write("<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>");
    oOut.write("<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>");
    oOut.write("</mo_prnciMaxCrowdingRadius>");
  }

  private static void WriteNCIMinNeighborDBH(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_nciMinNeighborDBH>");
    oOut.write("<mo_nmndVal species=\"Species_1\">10</mo_nmndVal>");
    oOut.write("<mo_nmndVal species=\"Species_2\">10</mo_nmndVal>");
    oOut.write("</mo_nciMinNeighborDBH>");
  }

  private static void WriteNCINeighborDBHEffect(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciNeighDBHEff>");
    oOut.write("<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>");
    oOut.write("<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>");
    oOut.write("</mo_prnciNeighDBHEff>");
  }

  private static void WriteNCINeighborDistEffect(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<mo_prnciNeighDistEff>");
    oOut.write("<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>");
    oOut.write("<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>");
    oOut.write("</mo_prnciNeighDistEff>");
  }

  private static void WriteNCIMaxProbSurvival(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciMaxPotentialSurvival>");
    oOut.write("<mo_pmpgVal species=\"Species_1\">0.5</mo_pmpgVal>");
    oOut.write("<mo_pmpgVal species=\"Species_2\">0.7</mo_pmpgVal>");
    oOut.write("</mo_prnciMaxPotentialSurvival>");
  }

  private static void WriteNCISizeSensNCI(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSizeSensNCI>");
    oOut.write("<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>");
    oOut.write("<mo_pssnVal species=\"Species_2\">0.36</mo_pssnVal>");
    oOut.write("</mo_prnciSizeSensNCI>");
  }

  private static void WriteNCISizeEffectMode(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSizeEffectMode>");
    oOut.write("<mo_psemVal species=\"Species_1\">34.24638994</mo_psemVal>");
    oOut.write("<mo_psemVal species=\"Species_2\">122.23</mo_psemVal>");
    oOut.write("</mo_prnciSizeEffectMode>");
  }

  private static void WriteNCISizeEffectVar(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSizeEffectVar>");
    oOut.write("<mo_psevVal species=\"Species_1\">5</mo_psevVal>");
    oOut.write("<mo_psevVal species=\"Species_2\">2.36</mo_psevVal>");
    oOut.write("</mo_prnciSizeEffectVar>");
  }

  private static void WriteNCISlope(FileWriter oOut) throws java.io.IOException {
    oOut.write("<mo_prnciNCISlope>");
    oOut.write("<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>");
    oOut.write("<mo_pnslVal species=\"Species_2\">4.57</mo_pnslVal>");
    oOut.write("</mo_prnciNCISlope>");
  }

  private static void WriteNCISteepness(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciNCISteepness>");
    oOut.write("<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>");
    oOut.write("<mo_pnstVal species=\"Species_2\">1.26</mo_pnstVal>");
    oOut.write("</mo_prnciNCISteepness>");
  }

  private static void WriteNCIStormEffMedDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciStormEffMedDmg>");
    oOut.write("<mo_psemdVal species=\"Species_1\">0.89</mo_psemdVal>");
    oOut.write("<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>");
    oOut.write("</mo_prnciStormEffMedDmg>");
  }

  private static void WriteNCIStormEffFullDmg(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciStormEffFullDmg>");
    oOut.write("<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>");
    oOut.write("<mo_psefdVal species=\"Species_2\">0.34</mo_psefdVal>");
    oOut.write("</mo_prnciStormEffFullDmg>");
  }

  private static void WriteNCINeighStormEffMedDmg(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<mo_prnciNeighStormEffMedDmg>");
    oOut.write("<mo_pnsemdVal species=\"Species_1\">0.83</mo_pnsemdVal>");
    oOut.write("<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>");
    oOut.write("</mo_prnciNeighStormEffMedDmg>");
  }

  private static void WriteNCINeighStormEffFullDmg(FileWriter oOut) throws java.
      io.
      IOException {
    oOut.write("<mo_prnciNeighStormEffFullDmg>");
    oOut.write("<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>");
    oOut.write("<mo_pnsefdVal species=\"Species_2\">0.27</mo_pnsefdVal>");
    oOut.write("</mo_prnciNeighStormEffFullDmg>");
  }

  private static void WriteNCILambdas(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mo_prnciSpecies_1NeighborLambda>");
    oOut.write("<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>");
    oOut.write("<mo_plVal species=\"Species_2\">0.71</mo_plVal>");
    oOut.write("</mo_prnciSpecies_1NeighborLambda>");
    oOut.write("<mo_prnciSpecies_2NeighborLambda>");
    oOut.write("<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>");
    oOut.write("<mo_plVal species=\"Species_2\">0.12</mo_plVal>");
    oOut.write("</mo_prnciSpecies_2NeighborLambda>");
  }

  private static void WriteValidFileEnd(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<mortality>");
    WriteNCIMaxCrowdingRadius(oOut);
    WriteNCINeighborDBHEffect(oOut);
    WriteNCINeighborDistEffect(oOut);
    WriteNCIMaxProbSurvival(oOut);
    WriteNCISizeSensNCI(oOut);
    WriteNCISizeEffectMode(oOut);
    WriteNCISizeEffectVar(oOut);
    WriteNCISlope(oOut);
    WriteNCISteepness(oOut);
    WriteNCIStormEffMedDmg(oOut);
    WriteNCIStormEffFullDmg(oOut);
    WriteNCINeighStormEffMedDmg(oOut);
    WriteNCINeighStormEffFullDmg(oOut);
    WriteNCIMinNeighborDBH(oOut);
    WriteNCILambdas(oOut);
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
  }
}
